#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
import copy
import datetime
import json
import os
import tempfile

from uge.config.config_manager import ConfigManager
from uge.exceptions.invalid_argument import InvalidArgument
from uge.exceptions.invalid_request import InvalidRequest


class QconfObject(object):
    """ This class encapsulates data and functionality common to all Qconf API objects. """

    VERSION = '1.0'
    NAME_KEY = None
    UGE_PYTHON_OBJECT_MAP = {
        'NONE': None,
        'INFINITY': float('inf'),
        'TRUE': True,
        'FALSE': False,
    }
    UGE_CASE_SENSITIVE_KEYS = {}
    USER_PROVIDED_KEYS = []
    REQUIRED_DATA_DEFAULTS = {}
    BOOL_KEY_MAP = {}
    INT_KEY_MAP = {}
    FLOAT_KEY_MAP = {}
    LIST_KEY_MAP = {}
    DEFAULT_LIST_DELIMITER = ','
    DICT_KEY_MAP = {}
    DEFAULT_DICT_DELIMITER = ','
    DICT_VALUE_DELIMITER = '='
    OPTIONAL_KEYS_ALLOWED = False

    def __init__(self, name=None, data=None, metadata=None, json_string=None):
        """ 
        Class constructor. 

        :param name: Object name. If provided, it will override object's name from data or JSON string parameters.
        :type name: str

        :param data: Object data. If provided, it will override corresponding data from object's JSON string representation.
        :type data: varies

        :param metadata: Object metadata. If provided, it will override corresponding metadata from object's JSON string representation.
        :type metadata: dict

        :param json_string: Object's JSON string representation.
        :type json_string: str

        :raises: **InvalidArgument** - in case metadata is not a dictionary, JSON string is not valid, or it does not contain dictionary representing a Qconf object.
        """
        self.name = name
        self.metadata = {}
        if not hasattr(self, 'data'):
            self.data = {}

        # Unpack and check json
        json_dict = self.unpack_input_json(json_string)
        if json_dict:
            if 'data' in json_dict:
                self.data = json_dict.get('data')
                del json_dict['data']
            self.metadata = json_dict

        # Merge json entries with provided metadata
        if metadata:
            self.check_input_metadata(metadata)
            self.metadata.update(metadata)

        # Merge json entries with provided data
        if data:
            self.check_input_data(data)
            if type(data) == dict:
                self.data.update(data)
            else:
                self.data = data

        if name and self.NAME_KEY:
            self.data[self.NAME_KEY] = name

        # Convert list and dict keys
        self.convert_list_keys()
        self.convert_dict_keys()

        # Add standard metadata
        self.metadata['object_version'] = self.VERSION
        self.metadata['object_class'] = self.__class__.__name__

    def unpack_input_json(self, json_string):
        if not json_string:
            return None
        try:
            json_dict = json.loads(json_string)
        except Exception as ex:
            raise InvalidArgument('Input is not a valid json string: %s (error: %s).' % (str(json_string), ex))
        if type(json_dict) != dict:
            raise InvalidArgument('Input json string does not contain dictionary: %s.' % str(json_string))
        return json_dict

    def check_user_provided_keys(self):
        """ 
        Checks for presence of all data keys that must be provided by user.

        :raises: **InvalidRequest** - in case object's data is not a dictionary, or if any of the required keys are missing.
        """
        if type(self.data) != dict:
            raise InvalidRequest('Data object is not a dictionary: %s.' % str(self.data))

        for key in self.USER_PROVIDED_KEYS:
            if not self.data.get(key):
                raise InvalidRequest('Input data is missing required object key: %s.' % str(key))

    def check_input_data(self, data):
        pass

    def check_input_metadata(self, metadata):
        if metadata:
            if type(metadata) != dict:
                raise InvalidArgument('Provided metadata is not a dictionary: %s.' % str(metadata))

    def remove_optional_keys(self):
        """ 
        Removes values for keys that are not required from object's data. 

        :raises: **InvalidRequest** - in case object's data is not a dictionary.
        """
        if self.OPTIONAL_KEYS_ALLOWED:
            return

        if type(self.data) != dict:
            raise InvalidRequest('Data object is not a dictionary: %s.' % str(self.data))

        removed_keys = []
        for (key, _) in list(self.data.items()):
            if key not in self.get_required_data_defaults():
                if key not in self.USER_PROVIDED_KEYS and not key.startswith('#'):
                    removed_keys.append(key)
        for key in removed_keys:
            del self.data[key]

    def update_with_required_data_defaults(self):
        """ 
        Updates object with default values for required data keys.

        :raises: **InvalidArgument** - in case object's data is not a dictionary.
        """
        if type(self.data) != dict:
            raise InvalidRequest('Data object is not a dictionary: %s.' % str(self.data))
        for (key, value) in list(self.get_required_data_defaults().items()):
            if key not in self.data:
                if type(value) == bytes:
                    for env_var in ['SGE_ROOT', 'SGE_CELL']:
                        value = value.replace(env_var, os.environ[env_var])
                self.data[key] = value

    def get_tmp_file(self):
        fd, tmp_file_path = tempfile.mkstemp(text=True)
        tmp_file = os.fdopen(fd, 'w')
        tmp_dir_path = None
        return (tmp_file, tmp_file_path, tmp_dir_path)

    def get_required_data_defaults(self):
        return self.REQUIRED_DATA_DEFAULTS

    def convert_list_keys(self):
        for key in list(self.LIST_KEY_MAP.keys()):
            value = self.data.get(key)
            if value is not None:
                if type(value) == bytes or type(value) == str:
                    delimiter = self.LIST_KEY_MAP.get(key, self.DEFAULT_LIST_DELIMITER)
                    self.data[key] = value.split(delimiter)
                elif type(value) != list:
                    raise InvalidArgument(
                        'Value for key %s must be provided either as a string, or as a python list of strings.' % key)

    def parse_value_as_dict(self, key, value):
        delimiter = self.DICT_KEY_MAP.get(key, self.DEFAULT_LIST_DELIMITER)
        items = value.split(delimiter)
        value_dict = {}
        for item in items:
            if item.find(self.DICT_VALUE_DELIMITER) < 0:
                raise InvalidArgument(
                    'Cannot parse dictionary value: Unexpected format of item %s for key %s.' % (item, key))
            item_tokens = item.split(self.DICT_VALUE_DELIMITER)
            item_key = item_tokens[0]
            item_value = self.DICT_VALUE_DELIMITER.join(item_tokens[1:])
            value_dict[item_key] = self.uge_to_py(item_key, item_value)
        return value_dict

    def convert_dict_keys(self):
        for key in list(self.DICT_KEY_MAP.keys()):
            value = self.data.get(key)
            if value is not None:
                if type(value) == bytes:
                    self.data[key] = self.parse_value_as_dict(key, value)
                elif type(value) != dict:
                    raise InvalidArgument(
                        'Value for key %s must be provided either as a string, or as a python dictionary.' % key)

    def set_data_dict_from_qconf_output(self, qconf_output):
        data = self.to_dict(qconf_output)
        self.data = data

    @classmethod
    def get_list_from_qconf_output(cls, qconf_output):
        qconf_output = qconf_output.strip()
        qconf_list = []
        if len(qconf_output):
            qconf_list = qconf_output.split('\n')
        return qconf_list

    @classmethod
    def get_bool_key_map(cls, key_map):
        bool_key_map = {}
        for (key, value) in list(key_map.items()):
            if type(value) == bool:
                bool_key_map[key] = value
            elif type(value) == dict:
                for (key2, value2) in list(value.items()):
                    if type(value2) == bool:
                        bool_key_map[key2] = value2
        return bool_key_map

    @classmethod
    def get_int_key_map(cls, key_map):
        int_key_map = {}
        for (key, value) in list(key_map.items()):
            if type(value) == int:
                int_key_map[key] = value
            elif type(value) == dict:
                for (key2, value2) in list(value.items()):
                    if type(value2) == int:
                        int_key_map[key2] = value2
        return int_key_map

    @classmethod
    def get_float_key_map(cls, key_map):
        float_key_map = {}
        for (key, value) in list(key_map.items()):
            if type(value) == float:
                float_key_map[key] = value
        return float_key_map

    @classmethod
    def get_list_key_map(cls, key_map):
        list_key_map = {}
        for (key, value) in list(key_map.items()):
            if type(value) == list:
                list_key_map[key] = value
        return list_key_map

    def uge_to_py(self, key, value):
        uppercase_value = value.upper()
        for (uge_value, py_value) in list(self.UGE_PYTHON_OBJECT_MAP.items()):
            if uge_value == uppercase_value:
                return py_value
        if key in self.LIST_KEY_MAP:
            # Key is designated as list key.
            # Try to split by corresponding delimiter.
            delimiter = self.LIST_KEY_MAP.get(key)
            if value.find(delimiter) > 0:
                return value.split(delimiter)
            return [value]
        elif key in self.DICT_KEY_MAP:
            # Key is designated as dict key.
            # Try to split by corresponding delimiter.
            return self.parse_value_as_dict(key, value)
        elif key in self.INT_KEY_MAP:
            try:
                return int(value)
            except:
                # We cannot convert this string to int
                pass
        elif key in self.FLOAT_KEY_MAP:
            try:
                return float(value)
            except:
                # We cannot convert this string to float
                pass
        elif value.find(self.DEFAULT_LIST_DELIMITER) > 0:
            return value.split(self.DEFAULT_LIST_DELIMITER)
        return value

    def py_to_uge(self, key, value):
        for (uge_value, py_value) in list(self.UGE_PYTHON_OBJECT_MAP.items()):
            if value == py_value and type(value) == type(py_value):
                if key in self.UGE_CASE_SENSITIVE_KEYS:
                    return self.UGE_CASE_SENSITIVE_KEYS[key](uge_value)
                return uge_value
        if type(value) == list:
            delimiter = self.LIST_KEY_MAP.get(key, self.DEFAULT_LIST_DELIMITER)
            return delimiter.join(value)
        elif type(value) == dict:
            delimiter = self.DICT_KEY_MAP.get(key, self.DEFAULT_DICT_DELIMITER)
            dict_tokens = []
            for (item_key, item_value) in list(value.items()):
                dict_tokens.append('%s%s%s' % (item_key, self.DICT_VALUE_DELIMITER, item_value))
            return delimiter.join(dict_tokens)
        return value

    def to_uge(self):
        """ 
        Converts object to string acceptable as input for UGE qconf command.

        :returns: Object's UGE-formatted string.
        """
        lines = ''
        for (key, value) in list(self.data.items()):
            lines += '%s %s\n' % (key, self.py_to_uge(key, value))
        return lines

    def convert_data_to_uge_keywords(self, data):
        for (key, value) in list(data.items()):
            data[key] = self.py_to_uge(key, value)

    def to_json(self, use_uge_keywords=False):
        """ 
        Converts object to JSON string.

        :param use_uge_keywords: if True, UGE keywords (e.g. 'NONE') are restored before conversion to JSON; otherwise, no changes are made to object's data. Default is False.
        :type mode: bool

        :returns: Object's JSON representation.
        """
        json_dict = copy.copy(self.metadata)
        data = copy.copy(self.data)
        if use_uge_keywords:
            self.convert_data_to_uge_keywords(data)
        json_dict['data'] = data
        return json.dumps(json_dict)

    def to_dict(self, input_string):
        lines = input_string.split('\n')
        object_data = {}
        for line in lines:
            if not line:
                continue
            key_value = line.split(' ')
            key = key_value[0]
            value = line.replace(key, '', 1).strip()
            object_data[key] = self.uge_to_py(key, value)
        return object_data

    def set_get_metadata(self):
        """ 
        Sets default object metadata (user/timestamp) for API get operations.
        """
        cm = ConfigManager.get_instance()
        retrieved_by = '%s@%s' % (cm['user'], cm['host'])
        self.metadata['retrieved_by'] = retrieved_by
        self.metadata['retrieved_on'] = datetime.datetime.now().isoformat()

    def set_modify_metadata(self):
        """ 
        Sets default object metadata (user/timestamp) for API modify operations.
        """
        cm = ConfigManager.get_instance()
        modified_by = '%s@%s' % (cm['user'], cm['host'])
        self.metadata['modified_by'] = modified_by
        self.metadata['modified_on'] = datetime.datetime.now().isoformat()

    def set_add_metadata(self):
        """ 
        Sets default object metadata (user/timestamp) for API add operations.
        """
        cm = ConfigManager.get_instance()
        created_by = '%s@%s' % (cm['user'], cm['host'])
        self.metadata['created_by'] = created_by
        self.metadata['created_on'] = datetime.datetime.now().isoformat()



#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
from uge.api.qconf_api import QconfApi
from uge.api.ar_api import AdvanceReservationApi



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
import imp
import json
import re
from uge.exceptions.qconf_exception import QconfException
from uge.exceptions.invalid_request import InvalidRequest
from .uge_release_object_map import UGE_RELEASE_OBJECT_MAP


class QconfObjectFactory(object):

    @classmethod
    def __get_object_base_module_name(cls, class_name):
        # This method relies on convention:
        #     ResourceQuotaSet=>resource_quota_set
        base_module_name = class_name[0].lower()
        for letter in class_name[1:]:
            if letter.isupper():
                base_module_name += '_%s' % letter.lower()
            else:
                base_module_name += letter
        return base_module_name

    @classmethod
    def __get_object_class_from_uge_version(cls, uge_version, class_name, base_module_name=None):
        if not uge_version:
            raise InvalidRequest('Cannot generate %s object: UGE version must be specified.' % class_name)
        uge_version = re.search(r'\d+.\d+.\d+', uge_version).group(0)
        if uge_version not in UGE_RELEASE_OBJECT_MAP:
            raise QconfException('Unsupported UGE version: %s.' % uge_version)
        release_map = UGE_RELEASE_OBJECT_MAP.get(uge_version)
        object_version = release_map.get(class_name)
        return cls.__get_object_class_from_object_version(object_version, class_name, base_module_name)

    @classmethod
    def __get_object_class_from_object_version(cls, object_version, class_name, base_module_name=None):
        if not object_version:
            raise InvalidRequest('Object version not supplied for class %s.' % (class_name))

        if base_module_name is None:
            base_module_name = cls.__get_object_base_module_name(class_name)

        module_name = ('%s_v%s' % (base_module_name, object_version)).replace('.', '_')
        module_file = '%s/%s.py' % ('/'.join(__file__.split('/')[:-1]), module_name)
        module = imp.load_source('uge.objects.%s' % module_name, module_file)
        object_class = getattr(module, class_name)
        return object_class

    @classmethod
    def __generate_object(cls, object_class, name, data, metadata, json_string, add_required_data):
        generated_object = object_class(name=name, data=data, metadata=metadata, json_string=json_string)
        if add_required_data:
            generated_object.update_with_required_data_defaults()
        return generated_object

    @classmethod
    def generate_object(cls, json_string, target_uge_version=None):
        object_dict = json.loads(json_string)
        class_name = object_dict['object_class']
        object_version = object_dict['object_version']
        object_class = cls.__get_object_class_from_object_version(object_version, class_name)
        generated_object = object_class(json_string=json_string)
        generated_object.update_with_required_data_defaults()
        if target_uge_version:
            object_class = cls.__get_object_class_from_uge_version(target_uge_version, class_name)
            generated_object = object_class(json_string=generated_object.to_json())
            generated_object.update_with_required_data_defaults()
        return generated_object

    @classmethod
    def generate_cluster_queue(cls, uge_version, name=None, data=None, metadata=None, json_string=None,
                               add_required_data=True):
        cluster_queue_class = cls.__get_object_class_from_uge_version(uge_version, 'ClusterQueue')
        cluster_queue = cluster_queue_class(name=name, data=data, metadata=metadata, json_string=json_string)
        if add_required_data:
            cluster_queue.update_with_required_data_defaults()
        return cluster_queue

    @classmethod
    def generate_execution_host(cls, uge_version, name=None, data=None, metadata=None, json_string=None,
                                add_required_data=True):
        execution_host_class = cls.__get_object_class_from_uge_version(uge_version, 'ExecutionHost')
        execution_host = execution_host_class(name=name, data=data, metadata=metadata, json_string=json_string)
        if add_required_data:
            execution_host.update_with_required_data_defaults()
        return execution_host

    @classmethod
    def generate_host_group(cls, uge_version, name=None, data=None, metadata=None, json_string=None,
                            add_required_data=True):
        host_group_class = cls.__get_object_class_from_uge_version(uge_version, 'HostGroup')
        host_group = host_group_class(name=name, data=data, metadata=metadata, json_string=json_string)
        if add_required_data:
            host_group.update_with_required_data_defaults()
        return host_group

    @classmethod
    def generate_parallel_environment(cls, uge_version, name=None, data=None, metadata=None, json_string=None,
                                      add_required_data=True):
        parallel_environment_class = cls.__get_object_class_from_uge_version(uge_version, 'ParallelEnvironment')
        parallel_environment = parallel_environment_class(name=name, data=data, metadata=metadata,
                                                          json_string=json_string)
        if add_required_data:
            parallel_environment.update_with_required_data_defaults()
        return parallel_environment

    @classmethod
    def generate_user(cls, uge_version, name=None, data=None, metadata=None, json_string=None, add_required_data=True):
        user_class = cls.__get_object_class_from_uge_version(uge_version, 'User')
        user = user_class(name=name, data=data, metadata=metadata, json_string=json_string)
        if add_required_data:
            user.update_with_required_data_defaults()
        return user

    @classmethod
    def generate_project(cls, uge_version, name=None, data=None, metadata=None, json_string=None,
                         add_required_data=True):
        project_class = cls.__get_object_class_from_uge_version(uge_version, 'Project')
        project = project_class(name=name, data=data, metadata=metadata, json_string=json_string)
        if add_required_data:
            project.update_with_required_data_defaults()
        return project

    @classmethod
    def generate_calendar(cls, uge_version, name=None, data=None, metadata=None, json_string=None,
                          add_required_data=True):
        object_class = cls.__get_object_class_from_uge_version(uge_version, 'Calendar')
        generated_object = cls.__generate_object(object_class, name=name, data=data, metadata=metadata,
                                                 json_string=json_string, add_required_data=add_required_data)
        return generated_object

    @classmethod
    def generate_checkpointing_environment(cls, uge_version, name=None, data=None, metadata=None, json_string=None,
                                           add_required_data=True):
        object_class = cls.__get_object_class_from_uge_version(uge_version, 'CheckpointingEnvironment')
        generated_object = cls.__generate_object(object_class, name=name, data=data, metadata=metadata,
                                                 json_string=json_string, add_required_data=add_required_data)
        return generated_object

    @classmethod
    def generate_access_list(cls, uge_version, name=None, data=None, metadata=None, json_string=None,
                             add_required_data=True):
        object_class = cls.__get_object_class_from_uge_version(uge_version, 'AccessList')
        generated_object = cls.__generate_object(object_class, name=name, data=data, metadata=metadata,
                                                 json_string=json_string, add_required_data=add_required_data)
        return generated_object

    @classmethod
    def generate_scheduler_configuration(cls, uge_version, name=None, data=None, metadata=None, json_string=None,
                                         add_required_data=True):
        object_class = cls.__get_object_class_from_uge_version(uge_version, 'SchedulerConfiguration')
        generated_object = object_class(data=data, metadata=metadata, json_string=json_string)
        if add_required_data:
            generated_object.update_with_required_data_defaults()
        return generated_object

    @classmethod
    def generate_job_class(cls, uge_version, name=None, data=None, metadata=None, json_string=None,
                           add_required_data=True):
        object_class = cls.__get_object_class_from_uge_version(uge_version, 'JobClass')
        generated_object = cls.__generate_object(object_class, name=name, data=data, metadata=metadata,
                                                 json_string=json_string, add_required_data=add_required_data)
        return generated_object

    @classmethod
    def generate_cluster_configuration(cls, uge_version, name=None, data=None, metadata=None, json_string=None,
                                       add_required_data=True):
        object_class = cls.__get_object_class_from_uge_version(uge_version, 'ClusterConfiguration')
        generated_object = cls.__generate_object(object_class, name=name, data=data, metadata=metadata,
                                                 json_string=json_string, add_required_data=add_required_data)
        return generated_object

    @classmethod
    def generate_complex_configuration(cls, uge_version, name=None, data=None, metadata=None, json_string=None,
                                       add_required_data=True):
        object_class = cls.__get_object_class_from_uge_version(uge_version, 'ComplexConfiguration')
        generated_object = object_class(data=data, metadata=metadata, json_string=json_string)
        if add_required_data:
            generated_object.update_with_required_data_defaults()
        return generated_object

    @classmethod
    def generate_resource_quota_set(cls, uge_version, name=None, data=None, metadata=None, json_string=None,
                                    add_required_data=True):
        object_class = cls.__get_object_class_from_uge_version(uge_version, 'ResourceQuotaSet')
        generated_object = cls.__generate_object(object_class, name=name, data=data, metadata=metadata,
                                                 json_string=json_string, add_required_data=add_required_data)
        return generated_object

    @classmethod
    def generate_share_tree(cls, uge_version, name=None, data=None, metadata=None, json_string=None,
                            add_required_data=True):
        object_class = cls.__get_object_class_from_uge_version(uge_version, 'ShareTree')
        generated_object = object_class(data=data, metadata=metadata, json_string=json_string)
        if add_required_data:
            generated_object.update_with_required_data_defaults()
        return generated_object


#############################################################################
# Testing.
if __name__ == '__main__':
    print(__file__)
    module_file = '%s/%s.py' % ('/'.join(__file__.split('/')[:-1]), 'share_tree_v1_0')
    print(module_file)
    module = imp.load_source('uge.objects.share_tree_v1_0', module_file)
    print(module)
    print(getattr(module, 'ShareTree'))
    sconf = QconfObjectFactory.generate_share_tree('8.4.0')
    print(sconf)
    print(sconf.VERSION)



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
import types
import tempfile
from .utils import needs_uge
from .utils import generate_random_string
from .utils import create_config_file
from .utils import load_values

from uge.api.qconf_api import QconfApi
from uge.config.config_manager import ConfigManager
from uge.log.log_manager import LogManager
from uge.exceptions.object_not_found import ObjectNotFound
from uge.exceptions.object_already_exists import ObjectAlreadyExists

create_config_file()
API = QconfApi()
PROJECT_NAME = '%s' % generate_random_string(6)
CONFIG_MANAGER = ConfigManager.get_instance()
LOG_MANAGER = LogManager.get_instance()
VALUES_DICT = load_values('test_values.json')
print(VALUES_DICT)


@needs_uge
def test_object_not_found():
    try:
        project = API.get_prj('__non_existent_project__')
        assert (False)
    except ObjectNotFound as ex:
        # ok
        pass


def test_generate_prj():
    project = API.generate_prj(PROJECT_NAME)
    assert (project.data['name'] == PROJECT_NAME)


def test_add_prj():
    try:
        project_list = API.list_prjs()
    except ObjectNotFound as ex:
        # no projects defined
        project_list = []
    project = API.add_prj(name=PROJECT_NAME)
    assert (project.data['name'] == PROJECT_NAME)
    project_list2 = API.list_prjs()
    assert (len(project_list2) == len(project_list) + 1)
    assert (project_list2.count(PROJECT_NAME) == 1)


def test_list_prjs():
    project_list = API.list_prjs()
    assert (project_list is not None)


def test_object_already_exists():
    try:
        project = API.add_prj(name=PROJECT_NAME)
        assert (False)
    except ObjectAlreadyExists as ex:
        # ok
        pass


def test_get_prj():
    project = API.get_prj(PROJECT_NAME)
    assert (project.data['name'] == PROJECT_NAME)


def test_generate_prj_from_json():
    prj = API.get_prj(PROJECT_NAME)
    json = prj.to_json()
    prj2 = API.generate_object(json)
    assert (prj2.__class__.__name__ == prj.__class__.__name__)
    for key in list(prj.data.keys()):
        v = prj.data[key]
        v2 = prj2.data[key]
        if type(v) == list:
            assert (len(v) == len(v2))
            for s in v:
                assert (v2.count(s) == 1)
        elif type(v) == dict:
            for key in list(v.keys()):
                assert (str(v[key]) == str(v2[key]))
        else:
            assert (str(v) == str(v2))


def test_modify_prj():
    project = API.get_prj(PROJECT_NAME)
    oticket = project.data['oticket']
    project = API.modify_prj(name=PROJECT_NAME, data={'oticket': oticket + 1})
    oticket2 = project.data['oticket']
    assert (oticket2 == oticket + 1)


def test_delete_prj():
    project_list = API.list_prjs()
    API.delete_prj(PROJECT_NAME)
    try:
        project_list2 = API.list_prjs()
    except ObjectNotFound as ex:
        # no projects defined
        project_list2 = []
    assert (len(project_list2) == len(project_list) - 1)
    assert (project_list2.count(PROJECT_NAME) == 0)


def test_get_prjs():
    prjl = API.list_prjs()
    print("Project Names: " + prjl)
    prjs = API.get_prjs()
    print("Projects: " + str(prjs))
    for prj in prjs:
        print("#############################################")
        print(prj.to_uge())
        assert (prj.data['name'] in prjl)


def test_write_prjs():
    try:
        tdir = tempfile.mkdtemp()
        print("*************************** " + tdir)
        prjs = API.get_prjs()
        for prj in prjs:
            print("Before #############################################")
            print(prj.to_uge())

        new_prjs = []
        prj_names = VALUES_DICT['prj_names']
        for name in prj_names:
            nprj = API.generate_prj(name=name)
            new_prjs.append(nprj)
        API.mk_prjs_dir(tdir)
        API.write_prjs(new_prjs, tdir)
        API.add_prjs_from_dir(tdir)
        API.modify_prjs_from_dir(tdir)
        prjs = API.get_prjs()
        for prj in prjs:
            print("After #############################################")
            print(prj.to_uge())

        prjs = API.list_prjs()
        for name in prj_names:
            assert (name in prjs)
            print("project found: " + name)

    finally:
        API.delete_prjs(prj_names)
        API.rm_prjs_dir(tdir)


def test_add_prjs():
    try:
        new_prjs = []
        prj_names = VALUES_DICT['prj_names']
        for name in prj_names:
            nprj = API.generate_prj(name=name)
            new_prjs.append(nprj)

        # print all projects currently in the cluster
        prjs = API.get_prjs()
        for prj in prjs:
            print("Before #############################################")
            print(prj.to_uge())

        # add projects
        API.add_prjs(new_prjs)
        API.modify_prjs(new_prjs)

        # print all projects currently in the cluster
        prjs = API.get_prjs()
        for prj in prjs:
            print("After #############################################")
            print(prj.to_uge())

        # check that projects have been added
        prjs = API.list_prjs()
        for name in prj_names:
            assert (name in prjs)
            print("project found: " + name)

    finally:
        API.delete_prjs(prj_names)


def test_modify_prjs():
    try:
        add_projects = []
        prj_names = VALUES_DICT['prj_names']
        # prj_names = ['tp1', 'tp2']
        for name in prj_names:
            nprj = API.generate_prj(name=name)
            add_projects.append(nprj)

        # print all projects currently in the cluster
        print("Before #############################################")
        prjs = API.get_prjs()
        for prj in prjs:
            print(prj.to_uge())

        # add projects
        API.add_prjs(add_projects)

        # modify added projects
        print("Before modify #############################################")
        prjs = API.get_prjs()
        for prj in prjs:
            if prj.data['name'] in prj_names:
                prj.data['oticket'] += 1
            else:
                print("project not found: " + prj.data['name'])
            print(prj.to_uge())
        API.modify_prjs(prjs)

        # check that projects have been changed
        print("After #############################################")
        prjs = API.get_prjs()
        for p in prjs:
            print(p.to_uge())
            if p.data['name'] in prj_names:
                print("project found: " + p.data['name'] + " with oticket=" + str(p.data['oticket']))
                # assert(p.data['oticket'] == 1)

    finally:
        API.delete_prjs(prj_names)



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
try:
    import exceptions
except ImportError:
    import builtins as exceptions
import json

from uge.constants import uge_status


class AdvanceReservationException(exceptions.Exception):
    """
    Base Qrdel exception class. 

    Error code: uge_status.UGE_ERROR

    Usage examples:
    
    >>> raise AdvanceReservationException(error_message, error_code)
  
    >>> raise AdvanceReservationException(args=error_message)

    >>> raise AdvanceReservationException(exception=exception_object)
    """

    def __init__(self, error='', error_code=uge_status.UGE_ERROR, **kwargs):
        """ 
        Class constructor. 

        :param error: Error message.
        :type error: str

        :param error_code: Error code.
        :type error_code: int

        :param kwargs: Keyword arguments, may contain 'args=error_message', 'exception=exception_object', or 'error_details=details'.
        """
        args = error
        if args == '':
            args = kwargs.get('args', '')
        ex = kwargs.get('exception', None)
        if ex is not None and isinstance(ex, exceptions.Exception):
            ex_args = "%s" % (ex)
            if args == '':
                args = ex_args
            else:
                args = "%s (%s)" % (args, ex_args)
        exceptions.Exception.__init__(self, args)
        self.error_code = error_code
        self.error_details = kwargs.get('error_details', None)

    def get_args(self):
        """ 
        :returns: Exception arguments. 
        """
        return self.args

    def get_error_code(self):
        """ 
        :returns: Exception error code. 
        """
        return self.error_code

    def get_error_message(self):
        """ 
        :returns: Exception error message. 
        """
        return "%s" % (self.args)

    def get_error_details(self):
        """ 
        :returns: Exception error details. 
        """
        return "%s" % (self.error_details)

    def to_dict(self):
        """ 
        :returns: Exception data dictionary.
        """
        return {'error_message': '%s' % self.args,
                'error_code': self.error_code,
                'error_details': self.error_details,
                'class_name': self.__class__.__name__}

    def to_json(self):
        """ 
        :returns: JSON string with exception data.
        """
        return json.dumps(self.to_dict())



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#

import tempfile
import types

from .utils import needs_uge
from .utils import generate_random_string
from .utils import create_config_file
from .utils import load_values

from uge.api.qconf_api import QconfApi
from uge.config.config_manager import ConfigManager
from uge.log.log_manager import LogManager
from uge.exceptions.object_not_found import ObjectNotFound
from uge.exceptions.object_already_exists import ObjectAlreadyExists

create_config_file()
API = QconfApi()
PE_NAME = '%s.q' % generate_random_string(6)
CONFIG_MANAGER = ConfigManager.get_instance()
LOG_MANAGER = LogManager.get_instance()
VALUES_DICT = load_values('test_values.json')
print(VALUES_DICT)


@needs_uge
def test_object_not_found():
    try:
        pe = API.get_pe('__non_existent_pe__')
        assert (False)
    except ObjectNotFound as ex:
        # ok
        pass


def test_generate_pe():
    pe = API.generate_pe(PE_NAME)
    assert (pe.data['pe_name'] == PE_NAME)


def test_add_pe():
    try:
        pel = API.list_pes()
    except ObjectNotFound as ex:
        # no pes defined
        pel = []
    pe = API.add_pe(name=PE_NAME)
    assert (pe.data['pe_name'] == PE_NAME)
    pel2 = API.list_pes()
    assert (len(pel2) == len(pel) + 1)
    assert (pel2.count(PE_NAME) == 1)


def test_list_pes():
    pel = API.list_pes()
    assert (pel is not None)


def test_object_already_exists():
    try:
        pe = API.add_pe(name=PE_NAME)
        assert (False)
    except ObjectAlreadyExists as ex:
        # ok
        pass


def test_get_pe():
    pe = API.get_pe(PE_NAME)
    assert (pe.data['pe_name'] == PE_NAME)


def test_generate_pe_from_json():
    pe = API.get_pe(PE_NAME)
    json = pe.to_json()
    pe2 = API.generate_object(json)
    assert (pe2.__class__.__name__ == pe.__class__.__name__)
    for key in list(pe.data.keys()):
        v = pe.data[key]
        v2 = pe2.data[key]
        if type(v) == list:
            assert (len(v) == len(v2))
            for s in v:
                assert (v2.count(s) == 1)
        elif type(v) == dict:
            for key in list(v.keys()):
                assert (str(v[key]) == str(v2[key]))
        else:
            assert (str(v) == str(v2))


def test_modify_pe():
    pe = API.get_pe(PE_NAME)
    slots = pe.data['slots']
    pe = API.modify_pe(name=PE_NAME, data={'slots': slots + 1})
    slots2 = pe.data['slots']
    assert (slots2 == slots + 1)


def test_get_acls():
    pel = API.list_pes()
    pes = API.get_pes()
    for pe in pes:
        print("#############################################")
        print(pe.to_uge())
        assert (pe.data['pe_name'] in pel)

def test_write_pes():
    try:
        tdir = tempfile.mkdtemp()
        print("*************************** " + tdir)
        pe_names = VALUES_DICT['pe_names']
        pes = API.get_pes()
        for pe in pes:
            print("Before #############################################")
            print(pe.to_uge())

        new_pes = []
        for name in pe_names:
            npe = API.generate_pe(name=name)
            new_pes.append(npe)
        API.mk_pes_dir(tdir)
        API.write_pes(new_pes, tdir)
        API.add_pes_from_dir(tdir)
        API.modify_pes_from_dir(tdir)
        pes = API.get_pes()
        for pe in pes:
            print("After #############################################")
            print(pe.to_uge())

        pes = API.list_pes()
        for name in pe_names:
            assert (name in pes)
            print("pe found: " + name)

    finally:
        API.delete_pes_from_dir(tdir)
        API.rm_pes_dir(tdir)


def test_add_pes():
    try:
        new_pes = []
        pe_names = VALUES_DICT['pe_names']
        for name in pe_names:
            npe = API.generate_pe(name=name)
            new_pes.append(npe)

        # print all pes currently in the cluster
        pes = API.get_pes()
        for pe in pes:
            print("Before #############################################")
            print(pe.to_uge())

        # add pes
        API.add_pes(new_pes)
        API.modify_pes(new_pes)

        # print all pes currently in the cluster
        pes = API.get_pes()
        for pe in pes:
            print("After #############################################")
            print(pe.to_uge())

        # check that cals have been added
        pes = API.list_pes()
        for name in pe_names:
            assert (name in pes)
            print("pe found: " + name)

    finally:
        API.delete_pes(new_pes)


def test_delete_pe():
    pel = API.list_pes()
    API.delete_pe(PE_NAME)
    try:
        pel2 = API.list_pes()
    except ObjectNotFound as ex:
        # no pes defined
        pel2 = []
    assert (len(pel2) == len(pel) - 1)
    assert (pel2.count(PE_NAME) == 0)



#!/usr/bin/env python
# 
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
# 
from nose import SkipTest

from .utils import needs_uge
from .utils import create_config_file

from uge.api.qconf_api import QconfApi
from uge.config.config_manager import ConfigManager
from uge.log.log_manager import LogManager
from uge.exceptions.object_not_found import ObjectNotFound
from uge.exceptions.object_already_exists import ObjectAlreadyExists

create_config_file()
API = QconfApi()
CONFIG_MANAGER = ConfigManager.get_instance()
LOG_MANAGER = LogManager.get_instance()


@needs_uge
def test_list_shosts():
    try:
        hl = API.list_shosts()
        assert (hl is not None)
    except ObjectNotFound as ex:
        raise SkipTest('There are no configured UGE submit hosts.')


def test_object_already_exists():
    try:
        try:
            hl = API.list_shosts()
            if len(hl):
                API.add_shosts([hl[0]])
                assert (False)
            else:
                raise SkipTest('There are no configured UGE submit hosts.')
        except ObjectNotFound as ex:
            raise SkipTest('There are no configured UGE submit hosts.')
    except ObjectAlreadyExists as ex:
        # ok
        pass


def test_delete_and_add_shosts():
    try:
        hl = API.list_shosts()
        if not len(hl):
            raise SkipTest('There are no configured UGE submit hosts.')
        host_name = hl[0]
        hl2 = API.delete_shosts([host_name])
        assert (hl2.data.count(host_name) == 0)
        hl3 = API.add_shosts(host_name)
        assert (hl3.data.count(host_name) == 1)
    except ObjectNotFound as ex:
        raise SkipTest('There are no configured UGE submit hosts.')



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#

import logging

from .trace_log_record import TraceLogRecord


class TraceLogger(logging.getLoggerClass()):
    # Use a level of 5...in logging module 0 = NOTSET and 10 = DEBUG
    TRACE = 5

    # Name to use when printing log level
    TRACE_NAME = 'TRACE'

    @classmethod
    def getLogLevel(cls, level_str):
        """ Return the int level that matches level_str if we know it """
        level = None
        if level_str == TraceLogger.TRACE_NAME:
            level = TraceLogger.TRACE
        return level

    @classmethod
    def register(cls):
        """ Register custom trace logger with the logging subsystem """

        # Register a new level / name mapping
        logging.addLevelName(TraceLogger.TRACE, TraceLogger.TRACE_NAME)

        # Override the default logging class for the python logger
        logging.setLoggerClass(TraceLogger)

    def trace(self, msg, *args, **kwargs):
        """ Trace call wrapping the logging.log method """
        if self.manager.disable >= TraceLogger.TRACE:
            return

        if TraceLogger.TRACE >= self.getEffectiveLevel():
            self.log(TraceLogger.TRACE, msg, *args, **kwargs)

    def makeRecord(self, name, level, fn, lno, msg, args, exc_info,
                   func=None, extra=None, stack_info=False):
        """
        Custom makeRecord so call stack variables can be updated with
        proper values
        """

        if level == TraceLogger.TRACE:
            return TraceLogRecord(
                name, level, fn, lno, msg, args, exc_info, func)

        return logging.Logger.makeRecord(
            self, name, level, fn, lno, msg, args, exc_info, func)



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
import os
import tempfile
from .qconf_object import QconfObject


class ClusterConfiguration(QconfObject):
    """ This class encapsulates UGE cluster configuration object. """

    #: Object version.
    VERSION = '1.0'

    #: Object name key.
    NAME_KEY = None

    #: Object keys that must be provided by user.
    USER_PROVIDED_KEYS = []

    #: Default values for required data keys for the global configuration.
    #: Value for execd_spool_dir key will depend on $SGE_ROOT and $SGE_CELL.
    REQUIRED_GLOBAL_DATA_DEFAULTS = {
        'execd_spool_dir': 'SGE_ROOT/SGE_CELL/spool',
        'mailer': '/bin/mail',
        'xterm': '/usr/bin/xterm',
        'load_sensor': None,
        'prolog': None,
        'epilog': None,
        'shell_start_mode': 'unix_behavior',
        'login_shells': ['sh', 'bash', 'ksh', 'csh', 'tcsh'],
        'min_uid': 0,
        'min_gid': 0,
        'user_lists': None,
        'xuser_lists': None,
        'projects': None,
        'xprojects': None,
        'default_jc': None,
        'enforce_jc': False,
        'enforce_project': False,
        'enforce_user': 'auto',
        'load_report_time': '00:00:40',
        'max_unheard': '00:04:00',
        'reschedule_unknown': '00:00:00',
        'loglevel': 'log_warning',
        'administrator_mail': None,
        'set_token_cmd': None,
        'pag_cmd': None,
        'token_extend_time': None,
        'shepherd_cmd': None,
        'qmaster_params': None,
        'execd_params': ['KEEP_ACTIVE=ERROR'],
        'reporting_params': {'accounting': True, 'reporting': False, 'flush_time': '00:00:13', 'joblog': False,
                             'sharelog': '00:00:00'},
        'finished_jobs': 0,
        'gid_range': '20000-20100',
        'qlogin_command': 'builtin',
        'qlogin_daemon': 'builtin',
        'rlogin_command': 'builtin',
        'rlogin_daemon': 'builtin',
        'rsh_command': 'builtin',
        'rsh_daemon': 'builtin',
        'max_aj_instances': 2000,
        'max_aj_tasks': 75000,
        'max_u_jobs': 0,
        'max_jobs': 0,
        'max_advance_reservations': 0,
        'auto_user_oticket': 0,
        'auto_user_fshare': 0,
        'auto_user_default_project': None,
        'auto_user_delete_time': 86400,
        'delegated_file_staging': False,
        'reprioritize': 0,
        'jsv_url': None,
        'jsv_allowed_mod': ['ac', 'h', 'i', 'e', 'o', 'j', 'M', 'N', 'p', 'w'],
        'cgroups_params': {'cgroup_path': None, 'cpuset': False, 'mount': False, 'freezer': False,
                           'freeze_pe_tasks': False, 'killing': False, 'forced_numa': False, 'h_vmem_limit': False,
                           'm_mem_free_hard': False, 'm_mem_free_soft': False, 'min_memory_limit': 0},
        'lost_job_timeout': '00:00:00',
        'enable_lost_job_reschedule': False,
    }

    #: Default values for required data keys for the host configuration.
    REQUIRED_HOST_DATA_DEFAULTS = {
        'mailer': '/bin/mail',
        'xterm': '/usr/bin/xterm',
    }

    BOOL_KEY_MAP = QconfObject.get_bool_key_map(REQUIRED_GLOBAL_DATA_DEFAULTS)
    INT_KEY_MAP = QconfObject.get_int_key_map(REQUIRED_GLOBAL_DATA_DEFAULTS)
    FLOAT_KEY_MAP = QconfObject.get_float_key_map(REQUIRED_GLOBAL_DATA_DEFAULTS)
    DEFAULT_LIST_DELIMITER = ','
    LIST_KEY_MAP = {
        'login_shells': ',',
        'user_lists': ',',
        'xuser_lists': ',',
        'projects': ',',
        'xprojects': ',',
        'qmaster_params': ',',
        'execd_params': ',',
        'jsv_allowed_mod': ',',
    }
    DEFAULT_DICT_DELIMITER = ' '
    DICT_KEY_MAP = {
        'reporting_params': ' ',
        'cgroups_params': ' ',
    }

    UGE_CASE_SENSITIVE_KEYS = {
    }

    def __init__(self, name='global', data=None, metadata=None, json_string=None):
        """ 
        Class constructor. 

        :param name: Configuration name (default: 'global').
        :type name: str

        :param data: Configuration data. If provided, it will override corresponding data from JSON string representation.
        :type data: dict

        :param metadata: Configuration metadata. If provided, it will override corresponding metadata from JSON string representation.
        :type metadata: dict

        :param json_string: Configuration JSON string representation.
        :type json_string: str

        :raises: **InvalidArgument** - in case metadata is not a dictionary, JSON string is not valid, or it does not contain dictionary representing a ClusterConfiguration object.
        """

        QconfObject.__init__(self, name=name, data=data, metadata=metadata, json_string=json_string)
        if not self.name:
            self.name = self.get_name_from_data()
            if not self.name:
                self.name = 'global'

    def get_name_from_data(self):
        for (key, _) in list(self.data.items()):
            if key.startswith('#'):
                return key[1:-1]  # remove comment and ending column characters
        return None

    def get_required_data_defaults(self):
        if self.name == 'global':
            return self.REQUIRED_GLOBAL_DATA_DEFAULTS
        return self.REQUIRED_HOST_DATA_DEFAULTS

    def get_tmp_file(self):
        tmp_dir_path = tempfile.mkdtemp()
        tmp_file_path = os.path.join(tmp_dir_path, self.name)
        tmp_file = open(tmp_file_path, 'w')
        return tmp_file, tmp_file_path, tmp_dir_path



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
from uge.cli.qconf_cli import QconfCli
from uge.api.qconf_api import QconfApi
from uge.exceptions.invalid_request import InvalidRequest


class QconfConvert(QconfCli):
    """ Qconf upgrade command. """

    def __init__(self):
        QconfCli.__init__(self)
        self.api = QconfApi()
        self.add_option('', '--input-file', dest='input_file',
                        help='Input file containing object\'s JSON representation.')
        self.add_option('', '--to-uge', dest='target_uge_version', default=self.api.get_uge_version(),
                        help='Target UGE version (default: %s).' % self.api.get_uge_version())
        self.add_option('', '--output-format', dest='output_format', default='json',
                        help='Output format, either json or uge (default: json).')

    def check_input_args(self):
        if not self.options.input_file:
            raise InvalidRequest('Missing input file.')
        if self.options.output_format not in ['json', 'uge']:
            raise InvalidRequest('Output format may be either json or uge.')

    def run_command(self):
        self.parse_args("""
    qconf-convert --input-file=INPUT_FILE
        [--to-uge=TARGET_UGE_VERSION]
        [--output-format=OUTPUT_FORMAT]

Description:
    Converts Qconf object with a given JSON representation to equivalent object corresponding to the specified target UGE version. 
""")
        input_file = self.options.input_file
        target_uge_version = self.options.target_uge_version
        json_string = open(input_file).read()
        qconf_object = self.api.generate_object(json_string, target_uge_version)
        if self.options.output_format == 'json':
            print(qconf_object.to_json())
        else:
            print(qconf_object.to_uge())


#############################################################################
# Run command.
def run():
    cli = QconfConvert()
    cli.run()


if __name__ == '__main__':
    run()



#!/usr/bin/env python
# 
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#

import tempfile
import types

from .utils import needs_uge
from .utils import generate_random_string
from .utils import create_config_file
from .utils import load_values

from uge.api.qconf_api import QconfApi
from uge.config.config_manager import ConfigManager
from uge.log.log_manager import LogManager
from uge.exceptions.object_not_found import ObjectNotFound
from uge.exceptions.object_already_exists import ObjectAlreadyExists

create_config_file()
API = QconfApi()
USER_NAME = '%s' % generate_random_string(6)
CONFIG_MANAGER = ConfigManager.get_instance()
LOG_MANAGER = LogManager.get_instance()
VALUES_DICT = load_values('test_values.json')
print(VALUES_DICT)


@needs_uge
def test_object_not_found():
    try:
        user = API.get_user('__non_existent_user__')
        assert (False)
    except ObjectNotFound as ex:
        # ok
        pass


def test_generate_user():
    user = API.generate_user(USER_NAME)
    assert (user.data['name'] == USER_NAME)


def test_add_user():
    try:
        userl = API.list_users()
    except ObjectNotFound as ex:
        # no users defined
        userl = []
    user = API.add_user(name=USER_NAME)
    assert (user.data['name'] == USER_NAME)
    userl2 = API.list_users()
    assert (len(userl2) == len(userl) + 1)
    assert (userl2.count(USER_NAME) == 1)


def test_list_users():
    userl = API.list_users()
    assert (userl is not None)


def test_object_already_exists():
    try:
        user = API.add_user(name=USER_NAME)
        assert (False)
    except ObjectAlreadyExists as ex:
        # ok
        pass


def test_get_user():
    user = API.get_user(USER_NAME)
    assert (user.data['name'] == USER_NAME)


def test_generate_user_from_json():
    user = API.get_user(USER_NAME)
    json = user.to_json()
    user2 = API.generate_object(json)
    assert (user2.__class__.__name__ == user.__class__.__name__)
    for key in list(user.data.keys()):
        v = user.data[key]
        v2 = user2.data[key]
        if type(v) == list:
            assert (len(v) == len(v2))
            for s in v:
                assert (v2.count(s) == 1)
        elif type(v) == dict:
            for key in list(v.keys()):
                assert (str(v[key]) == str(v2[key]))
        else:
            assert (str(v) == str(v2))


def test_modify_user():
    user = API.get_user(USER_NAME)
    oticket = user.data['oticket']
    user = API.modify_user(name=USER_NAME, data={'oticket': oticket + 1})
    oticket2 = user.data['oticket']
    assert (oticket2 == oticket + 1)


def test_get_users():
    userl = API.list_users()
    users = API.get_users()
    for user in users:
        print("#############################################")
        print(user.to_uge())
        assert (user.data['name'] in userl)


def test_write_users():
    try:
        tdir = tempfile.mkdtemp()
        print("*************************** " + tdir)
        user_names = VALUES_DICT['user_names']
        users = API.get_users()
        for user in users:
            print("Before #############################################")
            print(user.to_uge())

        new_users = []
        for name in user_names:
            nuser = API.generate_user(name=name)
            new_users.append(nuser)
        API.mk_users_dir(tdir)
        API.write_users(new_users, tdir)
        API.add_users_from_dir(tdir)
        API.modify_users_from_dir(tdir)
        users = API.get_users()
        for user in users:
            print("After #############################################")
            print(user.to_uge())

        users = API.list_users()
        for name in user_names:
            assert (name in users)
            print("user found: " + name)

    finally:
        API.delete_users_from_dir(tdir)
        API.rm_users_dir(tdir)


def test_add_users():
    try:
        new_users = []
        user_names = VALUES_DICT['user_names']
        for name in user_names:
            nuser = API.generate_user(name=name)
            new_users.append(nuser)

        # print all users currently in the cluster
        users = API.get_users()
        for user in users:
            print("Before #############################################")
            print(user.to_uge())

        # add user
        API.add_users(new_users)
        API.modify_users(new_users)

        # print all users currently in the cluster
        users = API.get_users()
        for user in users:
            print("After #############################################")
            print(user.to_uge())

        # check that users have been added
        users = API.list_users()
        for name in user_names:
            assert (name in users)
            print("user found: " + name)

    finally:
        API.delete_users(new_users)


def test_delete_user():
    userl = API.list_users()
    API.delete_user(USER_NAME)
    try:
        userl2 = API.list_users()
    except ObjectNotFound as ex:
        # no users defined
        userl2 = []
    assert (len(userl2) == len(userl) - 1)
    assert (userl2.count(USER_NAME) == 0)



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#

import logging
import re
import sys

from uge.config.config_manager import ConfigManager
from uge.exceptions.configuration_error import ConfigurationError
from uge.log.logger_factory import LoggerFactory
from uge.log.trace_logger import TraceLogger


class LogManager(object):
    """
    Singleton class for logging management.

    Usage:
        # Getting logger.
        from uge.log.log_manager import LogManager
        lm = LogManager.get_instance()
        logger = lm.get_logger('MyClass')

        # The following are different logging methods.
        logger.info('This is an info message')
        logger.debug('This is a debug message')
        logger.trace('This is a trace message')
        logger.warning('This is a warning message')
        logger.error('This is an error message')
        logger.critical('This is a critical error message')

        # The logger.exception() method should be called from an exception
        # handler only.
        try:
            myObject.someMethod()
        except Exception as ex:
            logger.exception('A bad thing happened.')

        # These methods are used to set user/system logging levels.
        lm.set_console_log_level('info')
        lm.set_file_log_level('debug')

    Configuration:
        The log manager class is initialized via a configuration file
        that may have the following sections:
            ConsoleLogging      # Used for output on the screen
            FileLogging         # Used for logging into a file

        If any of the above sections is missing from the configuration file,
        the correspondign handler is not instantiated (except for the
        console handler that is configured using predefined defaults).

        Each section in the configuration file should have the following
        keys:
            handler     # Indicates which handler class to use
            level       # Indicates logging level
            format      # Indicates format for log messages
            datefmt     # Indicates date format used for log messages

        Given below is an example of a valid configuration file:

            [ConsoleLogging]
            handler=StreamLogHandler(sys.stdout,)
            level=info
            format=[%(levelname)s] %(message)s
            datefmt=%m/%d/%y %H:%M:%S

            [RemoteLogging]
            handler=SocketLogHandler('localhost', 10020)
            level=notset
            format=%(asctime)s,%(msecs)d [%(levelname)s]
                %(module)s:%(lineno)d %(user)s@%(host)s %(name)s
                (%(process)d): %(message)s
            datefmt=%m/%d/%y %H:%M:%S

            [FileLogging]
            handler=TimedRotatingFileLogHandler('/tmp/uge.log')
            level=debug
            format=%(asctime)s,%(msecs)d [%(levelname)s]
                %(module)s:%(lineno)d %(user)s@%(host)s %(name)s
                (%(process)d): %(message)s
            datefmt=%m/%d/%y %H:%M:%S
    """

    # Singleton.
    __instance = None

    def __new__(cls, *args, **kwargs):
        # Allow subclasses to create their own instances.
        if cls.__instance is None or cls != type(cls.__instance):
            instance = object.__new__(cls, *args, **kwargs)
            instance.__init__()
            cls.__instance = instance
        return cls.__instance

    @classmethod
    def get_instance(cls, *args, **kwargs):
        return cls.__new__(cls, *args, **kwargs)

    def __init__(self):
        """ Initialize log manager instance. """
        # Only initialize once.
        if LogManager.__instance is not None:
            return
        self.user_handler_list = []
        self.system_handler_list = []

        self.loggerFactory = None

        # Register custom trace logger
        TraceLogger.register()
        self.__configure_handlers()

    def __get_log_level(self, level_str):
        # Get Log Level based on a string representation
        # level = logging.getLevelName(level_str)
        level = logging._checkLevel(level_str)
        # Level should be an integer
        try:
            return int(level)
        except ValueError:
            raise ValueError(
                '"%s" is not valid log/debug level' % (level_str))

    def __configure_handlers(self):
        """ Configure all log handlers from the config file. """
        cm = ConfigManager.get_instance()

        # Config manager will return the requested setting, or
        # its own default values if the default value is not passed in.
        handler = 'stream_log_handler.StreamLogHandler(sys.stderr,)'
        level = cm.get_console_log_level(None)
        format_ = cm.get_log_record_format(None)
        datefmt = cm.get_log_date_format(None)

        defaults = {
            'level': cm.get_console_log_level(),
            'format': cm.get_log_record_format(),
            'datefmt': cm.get_log_date_format(),
            'handler': handler
        }

        console_handler = None
        if cm.get_console_log_level() != 'notset':
            console_handler = self.__configure_handler(
                'ConsoleLogging', defaults, handler, level,
                format_, datefmt)

        if console_handler is not None:
            self.user_handler_list.append(console_handler)
            cm.set_console_log_level(console_handler.level)

        # Local file logging (only turned on if it is in the config file).
        handler = None
        level = cm.get_file_log_level(None)

        defaults['level'] = cm.get_file_log_level()
        defaults['handler'] = handler

        # Parse all of the file loggers present in the config file
        config_sections = cm.get_config_sections()
        for config_section in config_sections:
            if config_section.startswith('FileLogging'):
                file_handler = self.__configure_handler(
                    config_section, defaults, handler, level,
                    format_, datefmt)

                if file_handler is not None:
                    self.system_handler_list.append(file_handler)
                    cm.set_file_log_level(file_handler.level)

        # Remote logging (only turned on if it is in the config file).
        remote_handler = self.__configure_handler(
            'RemoteLogging', defaults, handler, level, format_,
            datefmt)

        if remote_handler is not None:
            self.system_handler_list.append(remote_handler)

        # Add handlers to the root logger.  Use logging class here
        # to make sure we can have a logger when we parse the
        # logger expressions
        root_logger = logging.getLogger('')

        for handler in self.user_handler_list + self.system_handler_list:
            root_logger.addHandler(handler)

        # Now get a logger factory based on our current config
        self.logger_factory = self.__create_logger_factory()

        # Reset levels.
        self.set_console_log_level(cm.get_console_log_level())
        self.set_file_log_level(cm.get_file_log_level())

    def __create_logger_factory(self):
        cm = ConfigManager.get_instance()
        root_log_level = cm.get_config_option('LoggerLevels', 'root', 'error')
        root_level_int = logging.getLevelName(root_log_level.upper())
        root_logger = logging.getLogger('')
        # root_logger.root.setLevel(root_level_int)
        root_logger.root.setLevel(40)
        root_logger.debug('Set root logger to %s' % root_level_int)
        expressions = cm.get_config_option('LoggerLevels', 'expressions')
        return LoggerFactory(expressions)

    def __configure_handler(self, config_section, defaults,
                            handler, level, format_, datefmt):
        """ Configure specified handler with a given defaults. """
        cm = ConfigManager.get_instance()
        cm.set_config_defaults(defaults)
        handler_option = cm.get_config_option(config_section,
                                              'handler', handler)

        # If handler_option is empty, handler cannot be instantiated.
        _handler = None
        if handler_option is not None:
            # Handler argument format: my_module.MyHandler(arg1, arg2, ...)
            # Module will be in lowercase letters, but the class
            # should be capitalized.
            handler_name = re.sub(r'\(.*', '', handler_option)
            module_name = handler_name.split('.')[0]
            try:
                exec('from uge.log import %s' % (module_name))
                exec('_handler = %s' % (handler_option))
            except IOError as ex:
                _errno = ex.errno
                import errno

                # If the exception raised is an I/O permissions error,
                # ignore it and disable this log handler.  This allows
                # non-root users to use the (system-wide) default log
                # configuration
                if _errno != errno.EACCES:
                    raise
                _handler = None
            except Exception as ex:
                raise ConfigurationError(exception=ex)

        # Only request setting from the config file if it was
        # not set via environment variable, or programmatically.
        if _handler is not None:
            try:
                _level = level
                if _level is None:
                    _level = cm.get_config_option(
                        config_section, 'level', defaults['level'])
                intLevel = self.__get_log_level(_level.upper())
                _handler.setLevel(intLevel)
                _handler.level = _level.upper()

                _format = format_
                if _format is None:
                    _format = cm.get_config_option(
                        config_section, 'format', defaults['format'])

                _datefmt = datefmt
                if _datefmt is None:
                    _datefmt = cm.get_config_option(
                        config_section, 'datefmt', defaults['datefmt'])

                _handler.setFormatter(logging.Formatter(_format, _datefmt))
            except Exception as ex:
                raise ConfigurationError(exception=ex)

            # Look to see if there is a filter to apply to the handler
            filter_ = None

            try:
                filter_ = cm.get_config_option(config_section, 'filter')
            except Exception as ex:
                pass

            if filter_:
                _handler.addFilter(logging.Filter(filter_))

        return _handler

    def get_logger(self, name):
        """ Get logger with a given name. """
        return self.logger_factory.get_logger(name)

    def set_console_log_level(self, level):
        """ Set user log level. """
        try:
            # We need to override the logger levels and the handler
            intLevel = self.__get_log_level(level.upper())
            for handler in self.user_handler_list:
                handler.setLevel(intLevel)
            self.logger_factory.force_level(intLevel)
        except Exception as ex:
            raise ConfigurationError(exception=ex)

    def set_file_log_level(self, level):
        """ Set system log level. """
        try:
            # We need to override the logger levels and the handler
            intLevel = self.__get_log_level(level.upper())
            for handler in self.system_handler_list:
                handler.setLevel(intLevel)
            self.logger_factory.force_level(intLevel)
        except Exception as ex:
            raise ConfigurationError(exception=ex)


#############################################################################
# Testing.
if __name__ == '__main__':
    lm = LogManager.get_instance()
    lm.set_console_log_level('trace')
    lm.set_file_log_level('trace')
    logger1 = lm.get_logger('log1')
    logger2 = lm.get_logger('log2')
    logger1.debug('Hello there, debug')
    logger2.trace('Hello there, trace')



#!/usr/bin/env python
# 
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
# 
import types
import copy
import random
from .utils import needs_uge
from .utils import generate_random_string
from .utils import create_config_file

from uge.api.qconf_api import QconfApi

create_config_file()
API = QconfApi()
CATTR_NAME = 'z%s' % generate_random_string(6)
CATTR_URGENCY = int(random.uniform(0, 100))


@needs_uge
def test_generate_cconf():
    cconf = API.generate_cconf()
    assert ('slots' in cconf.data)


def test_get_cconf():
    cconf = API.get_cconf()
    assert ('slots' in cconf.data)


def test_generate_cconf_from_json():
    cconf = API.get_cconf()
    json = cconf.to_json()
    cconf2 = API.generate_object(json)
    assert (cconf2.__class__.__name__ == cconf.__class__.__name__)
    for key in list(cconf.data.keys()):
        v = cconf.data[key]
        v2 = cconf2.data[key]
        if type(v) == list:
            assert (len(v) == len(v2))
            for s in v:
                assert (v2.count(s) == 1)
        elif type(v) == dict:
            for key in list(v.keys()):
                assert (str(v[key]) == str(v2[key]))
        else:
            assert (str(v) == str(v2))


def test_modify_cconf():
    cconf = API.get_cconf()
    slots = cconf.data['slots']
    urgency = slots['urgency']
    slots2 = copy.copy(slots)
    slots2['urgency'] = slots['urgency'] + 1
    cconf2 = API.modify_cconf(data={'slots': slots2})
    urgency2 = cconf2.data['slots']['urgency']
    assert (urgency2 == urgency + 1)
    cconf3 = API.modify_cconf(data={'slots': slots})
    urgency3 = cconf3.data['slots']['urgency']
    assert (urgency3 == urgency)


def test_add_cattr():
    cattr_data = {'shortcut': CATTR_NAME[0:3], 'type': 'INT', 'relop': '<=', 'requestable': True, 'consumable': True,
                  'default': 10, 'urgency': CATTR_URGENCY, 'aapre': False, 'affinity': 0.1,
                  'do_report': True, 'is_static': False}
    cconf = API.add_cattr(CATTR_NAME, cattr_data)
    assert (cconf.data[CATTR_NAME]['urgency'] == CATTR_URGENCY)


def test_modify_cattr():
    cconf = API.get_cconf()
    cattr_data = cconf.data[CATTR_NAME]
    assert (cattr_data['urgency'] == CATTR_URGENCY)
    cattr_data['urgency'] = 2 * CATTR_URGENCY
    cconf = API.modify_cattr(CATTR_NAME, cattr_data)
    assert (cconf.data[CATTR_NAME]['urgency'] == 2 * CATTR_URGENCY)


def test_delete_cattr():
    cconf = API.get_cconf()
    assert (CATTR_NAME in cconf.data)
    cconf = API.delete_cattr(CATTR_NAME)
    assert (CATTR_NAME not in cconf.data)



#!/usr/bin/env python
# 
# ___INFO__MARK_BEGIN__ 
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__ 
# 
import types
import random
from .utils import needs_uge
from .utils import generate_random_string
from .utils import create_config_file
from uge.exceptions.object_not_found import ObjectNotFound

from uge.api.qconf_api import QconfApi

create_config_file()
API = QconfApi()
ORIGINAL_STREE = None
PRJ1_NAME = 'prj%s' % generate_random_string(6)
PRJ1_SHARES = int(random.uniform(1,100))
PRJ2_NAME = 'prj%s' % generate_random_string(6)
PRJ2_SHARES = int(random.uniform(1,100))
PRJ3_NAME = 'prj%s' % generate_random_string(6)
PRJ3_SHARES = int(random.uniform(1,100))

@needs_uge
def test_stree_setup():
    try:
        ORIGINAL_STREE = API.get_stree()
        API.delete_stree()
    except ObjectNotFound:
        pass
    API.add_prj(name=PRJ1_NAME)
    API.add_prj(name=PRJ2_NAME)
    API.add_prj(name=PRJ3_NAME)

def test_generate_stree():
    stree = API.generate_stree(data=[
        {'id' : 0, 'name' : 'Root', 'type' : 0, 'shares' : 1, 'childnodes' : '1,2'}, 
        {'id' : 1, 'name' : PRJ1_NAME, 'shares' : PRJ1_SHARES}, 
        {'id' : 2, 'name' : PRJ2_NAME, 'shares' : PRJ2_SHARES}])
    assert(len(stree.data) == 3)
    assert(stree.data[1]['shares'] == PRJ1_SHARES)
    assert(stree.data[2]['shares'] == PRJ2_SHARES)

def test_add_stree():
    stree = API.add_stree(data=[
        {'id' : 0, 'name' : 'Root', 'type' : 0, 'shares' : 1, 'childnodes' : '1,2'}, 
        {'id' : 1, 'name' : PRJ1_NAME, 'shares' : PRJ1_SHARES}, 
        {'id' : 2, 'name' : PRJ2_NAME, 'shares' : PRJ2_SHARES}])
    assert(len(stree.data) == 3)
    assert(stree.data[0]['id'] == 0)
    assert(stree.data[1]['id'] == 1)
    assert(stree.data[1]['shares'] == PRJ1_SHARES)
    assert(stree.data[2]['id'] == 2)
    assert(stree.data[2]['shares'] == PRJ2_SHARES)

def test_get_stree():
    stree = API.get_stree()
    assert(len(stree.data) == 3)
    assert(stree.data[0]['id'] == 0)
    assert(stree.data[1]['id'] == 1)
    assert(stree.data[1]['shares'] == PRJ1_SHARES)
    assert(stree.data[2]['id'] == 2)
    assert(stree.data[2]['shares'] == PRJ2_SHARES)

def test_generate_stree_from_json():
    stree = API.get_stree()
    json = stree.to_json()
    stree2 = API.generate_object(json)
    assert(stree2.__class__.__name__ == stree.__class__.__name__)
    assert(len(stree.data) == len(stree2.data))
    for i in range(0,len(stree.data)):
        v = stree.data[i]
        v2 = stree2.data[i]
        assert(type(v) == dict)
        assert(type(v2) == dict)
        for key in list(v.keys()):
            x = v[key]
            x2 = v2[key]
            if type(x) == list:
                assert(len(x) == len(x2))
                for y in x:
                    assert(x2.count(y) == 1)
            else:
                assert(str(x) == str(x2))

def test_modify_stree():
    stree = API.get_stree()
    prj1_shares = stree.data[1]['shares']
    assert(prj1_shares == PRJ1_SHARES)
    stree.data[1]['shares'] = prj1_shares*2
    stree2 = API.modify_stree(stree)
    prj2_shares = stree2.data[1]['shares']
    assert(prj2_shares == 2*PRJ1_SHARES)

def test_add_stnode():
    stree = API.get_stree()
    stree_len = len(stree.data)
    stree2 = API.add_stnode('/%s' % PRJ3_NAME, PRJ3_SHARES)
    stree_len2 = len(stree2.data)
    assert(stree_len2 == stree_len+1)
    prj3_shares = stree2.data[-1]['shares']
    assert(prj3_shares == PRJ3_SHARES)

def test_delete_stnode():
    stree = API.get_stree()
    stree_len = len(stree.data)
    stree2 = API.delete_stnode('/%s' % PRJ3_NAME)
    stree_len2 = len(stree2.data)
    assert(stree_len2 == stree_len-1)

def test_delete_stree():
    API.delete_stree()
    try:
        stree = API.get_stree()
        assert(False)
    except ObjectNotFound as ex:
        pass

def test_modify_or_add_stree():
    stree = API.modify_or_add_stree(data=[
        {'id' : 0, 'name' : 'Root', 'type' : 0, 'shares' : 1, 'childnodes' : '1,2'}, 
        {'id' : 1, 'name' : PRJ1_NAME, 'shares' : PRJ1_SHARES}, 
        {'id' : 2, 'name' : PRJ2_NAME, 'shares' : PRJ2_SHARES}])
    assert(len(stree.data) == 3)
    assert(stree.data[0]['id'] == 0)
    assert(stree.data[1]['id'] == 1)
    assert(stree.data[1]['shares'] == PRJ1_SHARES)
    assert(stree.data[2]['id'] == 2)
    assert(stree.data[2]['shares'] == PRJ2_SHARES)

    stree = API.get_stree()
    prj1_shares = stree.data[1]['shares']
    assert(prj1_shares == PRJ1_SHARES)
    stree.data[1]['shares'] = prj1_shares*2
    stree2 = API.modify_or_add_stree(stree)
    prj2_shares = stree2.data[1]['shares']
    assert(prj2_shares == 2*PRJ1_SHARES)

def test_delete_stree_if_exists():
    # Delete share tree twice so we test case
    # when there is no share three defined
    API.delete_stree_if_exists()
    API.delete_stree_if_exists()

def test_get_stree_if_exists():
    stree = API.get_stree_if_exists()
    assert(len(stree.data) == 0)

def test_stree_cleanup():
    if ORIGINAL_STREE:
        API.add_stree(ORIGINAL_STREE)
    API.delete_prj(name=PRJ1_NAME)
    API.delete_prj(name=PRJ2_NAME)
    API.delete_prj(name=PRJ3_NAME)




#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
from uge.objects.complex_configuration_base import ComplexConfigurationBase


class ComplexConfiguration(ComplexConfigurationBase):
    """ This class encapsulates UGE complex configuration object. """

    #: Object version.
    VERSION = '4.0'

    #: Object name key.
    NAME_KEY = None

    #: Object keys that must be provided by user.
    USER_PROVIDED_KEYS = []

    #: Default values for required data keys.
    REQUIRED_DATA_DEFAULTS = {
        'arch': {'shortcut': 'a', 'type': 'RESTRING', 'relop': '==', 'requestable': True, 'consumable': False,
                 'default': None, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                 'do_report': True, 'is_static': True},
        'calendar': {'shortcut': 'c', 'type': 'RESTRING', 'relop': '==', 'requestable': True, 'consumable': False,
                     'default': None, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                     'do_report': False, 'is_static': False},
        'cpu': {'shortcut': 'cpu', 'type': 'DOUBLE', 'relop': '>=', 'requestable': True, 'consumable': False,
                'default': 0, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                'do_report': True, 'is_static': False},
        'd_rt': {'shortcut': 'd_rt', 'type': 'TIME', 'relop': '<=', 'requestable': True, 'consumable': False,
                 'default': '0:0:0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                 'do_report': False, 'is_static': False},
        'display_win_gui': {'shortcut': 'dwg', 'type': 'BOOL', 'relop': '==', 'requestable': True, 'consumable': False,
                            'default': 0, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                            'do_report': False, 'is_static': False},
        'docker': {'shortcut': 'dock', 'type': 'BOOL', 'relop': '==', 'requestable': True, 'consumable': False,
                   'default': 0, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                   'do_report': True, 'is_static': False},
        'docker_images': {'shortcut': 'dockimg', 'type': 'RESTRING', 'relop': '==', 'requestable': True,
                          'consumable': False, 'default': None, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                          'do_report': True, 'is_static': False},
        'h_core': {'shortcut': 'h_core', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': False,
                   'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                   'do_report': False, 'is_static': False},
        'h_cpu': {'shortcut': 'h_cpu', 'type': 'TIME', 'relop': '<=', 'requestable': True, 'consumable': False,
                  'default': '0:0:0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                  'do_report': False, 'is_static': False},
        'h_data': {'shortcut': 'h_data', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': False,
                   'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                   'do_report': False, 'is_static': False},
        'h_fsize': {'shortcut': 'h_fsize', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': False,
                    'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                    'do_report': False, 'is_static': False},
        'h_rss': {'shortcut': 'h_rss', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': False,
                  'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                  'do_report': False, 'is_static': False},
        'h_rt': {'shortcut': 'h_rt', 'type': 'TIME', 'relop': '<=', 'requestable': True, 'consumable': False,
                 'default': '0:0:0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                 'do_report': False, 'is_static': False},
        'h_stack': {'shortcut': 'h_stack', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': False,
                    'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                    'do_report': False, 'is_static': False},
        'h_vmem': {'shortcut': 'h_vmem', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': False,
                   'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                   'do_report': False, 'is_static': False},
        'hostname': {'shortcut': 'h', 'type': 'HOST', 'relop': '==', 'requestable': True, 'consumable': False,
                     'default': None, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                     'do_report': False, 'is_static': False},
        'load_avg': {'shortcut': 'la', 'type': 'DOUBLE', 'relop': '>=', 'requestable': False, 'consumable': False,
                     'default': 0, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                     'do_report': True, 'is_static': False},
        'load_long': {'shortcut': 'll', 'type': 'DOUBLE', 'relop': '>=', 'requestable': False, 'consumable': False,
                      'default': 0, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                      'do_report': True, 'is_static': False},
        'load_medium': {'shortcut': 'lm', 'type': 'DOUBLE', 'relop': '>=', 'requestable': False, 'consumable': False,
                        'default': 0, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                        'do_report': True, 'is_static': False},
        'load_short': {'shortcut': 'ls', 'type': 'DOUBLE', 'relop': '>=', 'requestable': False, 'consumable': False,
                       'default': 0, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                       'do_report': True, 'is_static': False},
        'm_cache_l1': {'shortcut': 'mcache1', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': False,
                       'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                       'do_report': True, 'is_static': True},
        'm_cache_l2': {'shortcut': 'mcache2', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': False,
                       'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                       'do_report': True, 'is_static': True},
        'm_cache_l3': {'shortcut': 'mcache3', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': False,
                       'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                       'do_report': True, 'is_static': True},
        'm_core': {'shortcut': 'core', 'type': 'INT', 'relop': '<=', 'requestable': True, 'consumable': False,
                   'default': 0, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                   'do_report': True, 'is_static': True},
        'm_gpu': {'shortcut': 'mgpu', 'type': 'INT', 'relop': '<=', 'requestable': True, 'consumable': False,
                   'default': 0, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                   'do_report': True, 'is_static': True},
        'm_mem_free': {'shortcut': 'mfree', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': True,
                       'default': '0', 'urgency': 0, 'aapre': True, 'affinity': 0.0,
                       'do_report': True, 'is_static': False},
        'm_mem_free_n0': {'shortcut': 'mfree0', 'type': 'MEMORY', 'relop': '<=', 'requestable': True,
                          'consumable': True, 'default': '0', 'urgency': 0, 'aapre': True, 'affinity': 0.0,
                          'do_report': True, 'is_static': False},
        'm_mem_free_n1': {'shortcut': 'mfree1', 'type': 'MEMORY', 'relop': '<=', 'requestable': True,
                          'consumable': True, 'default': '0', 'urgency': 0, 'aapre': True, 'affinity': 0.0,
                          'do_report': True, 'is_static': False},
        'm_mem_free_n2': {'shortcut': 'mfree2', 'type': 'MEMORY', 'relop': '<=', 'requestable': True,
                          'consumable': True, 'default': '0', 'urgency': 0, 'aapre': True, 'affinity': 0.0,
                          'do_report': True, 'is_static': False},
        'm_mem_free_n3': {'shortcut': 'mfree3', 'type': 'MEMORY', 'relop': '<=', 'requestable': True,
                          'consumable': True, 'default': '0', 'urgency': 0, 'aapre': True, 'affinity': 0.0,
                          'do_report': True, 'is_static': False},
        'm_mem_total': {'shortcut': 'mtotal', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': True,
                        'default': '0', 'urgency': 0, 'aapre': True, 'affinity': 0.0,
                        'do_report': True, 'is_static': True},
        'm_mem_total_n0': {'shortcut': 'mmem0', 'type': 'MEMORY', 'relop': '<=', 'requestable': True,
                           'consumable': True, 'default': '0', 'urgency': 0, 'aapre': True, 'affinity': 0.0,
                           'do_report': True, 'is_static': True},
        'm_mem_total_n1': {'shortcut': 'mmem1', 'type': 'MEMORY', 'relop': '<=', 'requestable': True,
                           'consumable': True, 'default': '0', 'urgency': 0, 'aapre': True, 'affinity': 0.0,
                           'do_report': True, 'is_static': True},
        'm_mem_total_n2': {'shortcut': 'mmem2', 'type': 'MEMORY', 'relop': '<=', 'requestable': True,
                           'consumable': True, 'default': '0', 'urgency': 0, 'aapre': True, 'affinity': 0.0,
                           'do_report': True, 'is_static': True},
        'm_mem_total_n3': {'shortcut': 'mmem3', 'type': 'MEMORY', 'relop': '<=', 'requestable': True,
                           'consumable': True, 'default': '0', 'urgency': 0, 'aapre': True, 'affinity': 0.0,
                           'do_report': True, 'is_static': True},
        'm_mem_used': {'shortcut': 'mused', 'type': 'MEMORY', 'relop': '>=', 'requestable': True, 'consumable': False,
                       'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                       'do_report': True, 'is_static': False},
        'm_mem_used_n0': {'shortcut': 'mused0', 'type': 'MEMORY', 'relop': '>=', 'requestable': True,
                          'consumable': False, 'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                          'do_report': True, 'is_static': False},
        'm_mem_used_n1': {'shortcut': 'mused1', 'type': 'MEMORY', 'relop': '>=', 'requestable': True,
                          'consumable': False, 'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                          'do_report': True, 'is_static': False},
        'm_mem_used_n2': {'shortcut': 'mused2', 'type': 'MEMORY', 'relop': '>=', 'requestable': True,
                          'consumable': False, 'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                          'do_report': True, 'is_static': False},
        'm_mem_used_n3': {'shortcut': 'mused3', 'type': 'MEMORY', 'relop': '>=', 'requestable': True,
                          'consumable': False, 'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                          'do_report': True, 'is_static': False},
        'm_numa_nodes': {'shortcut': 'nodes', 'type': 'INT', 'relop': '<=', 'requestable': True, 'consumable': False,
                         'default': 0, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                         'do_report': True, 'is_static': True},
        'm_socket': {'shortcut': 'socket', 'type': 'INT', 'relop': '<=', 'requestable': True, 'consumable': False,
                     'default': 0, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                     'do_report': True, 'is_static': True},
        'm_thread': {'shortcut': 'thread', 'type': 'INT', 'relop': '<=', 'requestable': True, 'consumable': False,
                     'default': 0, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                     'do_report': True, 'is_static': True},
        'm_topology': {'shortcut': 'topo', 'type': 'RESTRING', 'relop': '==', 'requestable': True, 'consumable': False,
                       'default': None, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                       'do_report': True, 'is_static': True},
        'm_topology_inuse': {'shortcut': 'utopo', 'type': 'RESTRING', 'relop': '==', 'requestable': True,
                             'consumable': False, 'default': None, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                             'do_report': True, 'is_static': True},
        'm_topology_numa': {'shortcut': 'unuma', 'type': 'RESTRING', 'relop': '==', 'requestable': True,
                            'consumable': False, 'default': None, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                            'do_report': True, 'is_static': True},
        'mem_free': {'shortcut': 'mf', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': False,
                     'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                     'do_report': True, 'is_static': False},
        'mem_total': {'shortcut': 'mt', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': False,
                      'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                      'do_report': True, 'is_static': True},
        'mem_used': {'shortcut': 'mu', 'type': 'MEMORY', 'relop': '>=', 'requestable': True, 'consumable': False,
                     'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                     'do_report': True, 'is_static': False},
        'min_cpu_interval': {'shortcut': 'mci', 'type': 'TIME', 'relop': '<=', 'requestable': False,
                             'consumable': False, 'default': '0:0:0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                             'do_report': False, 'is_static': False},
        'np_load_avg': {'shortcut': 'nla', 'type': 'DOUBLE', 'relop': '>=', 'requestable': False, 'consumable': False,
                        'default': 0, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                        'do_report': True, 'is_static': False},
        'np_load_long': {'shortcut': 'nll', 'type': 'DOUBLE', 'relop': '>=', 'requestable': False, 'consumable': False,
                         'default': 0, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                         'do_report': True, 'is_static': False},
        'np_load_medium': {'shortcut': 'nlm', 'type': 'DOUBLE', 'relop': '>=', 'requestable': False,
                           'consumable': False, 'default': 0, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                           'do_report': True, 'is_static': False},
        'np_load_short': {'shortcut': 'nls', 'type': 'DOUBLE', 'relop': '>=', 'requestable': False, 'consumable': False,
                          'default': 0, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                          'do_report': True, 'is_static': False},
        'num_proc': {'shortcut': 'p', 'type': 'INT', 'relop': '==', 'requestable': True, 'consumable': False,
                     'default': 0, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                     'do_report': True, 'is_static': True},
        'qname': {'shortcut': 'q', 'type': 'RESTRING', 'relop': '==', 'requestable': True, 'consumable': False,
                  'default': None, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                  'do_report': False, 'is_static': False},
        'rerun': {'shortcut': 're', 'type': 'BOOL', 'relop': '==', 'requestable': False, 'consumable': False,
                  'default': 0, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                  'do_report': False, 'is_static': False},
        's_core': {'shortcut': 's_core', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': False,
                   'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                   'do_report': False, 'is_static': False},
        's_cpu': {'shortcut': 's_cpu', 'type': 'TIME', 'relop': '<=', 'requestable': True, 'consumable': False,
                  'default': '0:0:0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                  'do_report': False, 'is_static': False},
        's_data': {'shortcut': 's_data', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': False,
                   'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                   'do_report': False, 'is_static': False},
        's_fsize': {'shortcut': 's_fsize', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': False,
                    'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                    'do_report': False, 'is_static': False},
        's_rss': {'shortcut': 's_rss', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': False,
                  'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                  'do_report': False, 'is_static': False},
        's_rt': {'shortcut': 's_rt', 'type': 'TIME', 'relop': '<=', 'requestable': True, 'consumable': False,
                 'default': '0:0:0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                 'do_report': False, 'is_static': False},
        's_stack': {'shortcut': 's_stack', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': False,
                    'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                    'do_report': False, 'is_static': False},
        's_vmem': {'shortcut': 's_vmem', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': False,
                   'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                   'do_report': False, 'is_static': False},
        'seq_no': {'shortcut': 'seq', 'type': 'INT', 'relop': '==', 'requestable': False, 'consumable': False,
                   'default': 0, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                   'do_report': False, 'is_static': False},
        'slots': {'shortcut': 's', 'type': 'INT', 'relop': '<=', 'requestable': True, 'consumable': True, 'default': 1,
                  'urgency': 1000, 'aapre': True, 'affinity': 0.0,
                  'do_report': False, 'is_static': False},
        'swap_free': {'shortcut': 'sf', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': False,
                      'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                      'do_report': True, 'is_static': False},
        'swap_rate': {'shortcut': 'sr', 'type': 'MEMORY', 'relop': '>=', 'requestable': True, 'consumable': False,
                      'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                      'do_report': True, 'is_static': False},
        'swap_rsvd': {'shortcut': 'srsv', 'type': 'MEMORY', 'relop': '>=', 'requestable': True, 'consumable': False,
                      'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                      'do_report': True, 'is_static': False},
        'swap_total': {'shortcut': 'st', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': False,
                       'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                       'do_report': True, 'is_static': True},
        'swap_used': {'shortcut': 'su', 'type': 'MEMORY', 'relop': '>=', 'requestable': True, 'consumable': False,
                      'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                      'do_report': True, 'is_static': False},
        'tmpdir': {'shortcut': 'tmp', 'type': 'RESTRING', 'relop': '==', 'requestable': False, 'consumable': False,
                   'default': None, 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                   'do_report': False, 'is_static': False},
        'virtual_free': {'shortcut': 'vf', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': False,
                         'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                         'do_report': True, 'is_static': False},
        'virtual_total': {'shortcut': 'vt', 'type': 'MEMORY', 'relop': '<=', 'requestable': True, 'consumable': False,
                          'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                          'do_report': True, 'is_static': True},
        'virtual_used': {'shortcut': 'vu', 'type': 'MEMORY', 'relop': '>=', 'requestable': True, 'consumable': False,
                         'default': '0', 'urgency': 0, 'aapre': False, 'affinity': 0.0,
                         'do_report': True, 'is_static': False},
    }

    def __init__(self, data=None, metadata=None, json_string=None):
        """ 
        Class constructor. 

        :param data: Configuration data. If provided, it will override corresponding data from JSON string representation.
        :type data: dict

        :param metadata: Configuration metadata. If provided, it will override corresponding metadata from JSON string representation.
        :type metadata: dict

        :param json_string: Configuration JSON string representation.
        :type json_string: str

        :raises: **InvalidArgument** - in case metadata is not a dictionary, JSON string is not valid, or it does not contain dictionary representing a ClusterConfiguration object.
        """

        ComplexConfigurationBase.__init__(self, data=data, metadata=metadata, json_string=json_string)



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
import copy

UGE_RELEASE_OBJECT_MAP = {}

# 8.3.1p9
UGE_RELEASE_OBJECT_MAP['8.3.1p9'] = {
    'AccessList'               : '1.0',
    'Calendar'                 : '1.0',
    'CheckpointingEnvironment' : '1.0',
    'ClusterConfiguration'     : '1.0',
    'ClusterQueue'             : '1.0',
    'ComplexConfiguration'     : '1.0',
    'ExecutionHost'            : '1.0',
    'HostGroup'                : '1.0',
    'JobClass'                 : '1.0',
    'ParallelEnvironment'      : '1.0',
    'Project'                  : '1.0',
    'ResourceQuotaSet'         : '1.0',
    'SchedulerConfiguration'   : '1.0',
    'ShareTree'                : '1.0',
    'User'                     : '1.0',
}

# 8.3.1p12
UGE_RELEASE_OBJECT_MAP['8.3.1p12'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.3.1p9'])

# 8.4.0
UGE_RELEASE_OBJECT_MAP['8.4.0'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.3.1p9'])
UGE_RELEASE_OBJECT_MAP['8.4.0']['ComplexConfiguration'] = '2.0'

# 8.4.3
UGE_RELEASE_OBJECT_MAP['8.4.3'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.4.0'])

# 8.4.4
UGE_RELEASE_OBJECT_MAP['8.4.4'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.4.0'])

# 8.4.5
UGE_RELEASE_OBJECT_MAP['8.4.5'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.4.0'])

# 8.5.0
UGE_RELEASE_OBJECT_MAP['8.5.0'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.4.0'])
UGE_RELEASE_OBJECT_MAP['8.5.0']['ClusterConfiguration'] = '2.0'
UGE_RELEASE_OBJECT_MAP['8.5.0']['JobClass'] = '2.0'

# 8.5.1
UGE_RELEASE_OBJECT_MAP['8.5.1'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.5.0'])

# 8.5.2
UGE_RELEASE_OBJECT_MAP['8.5.2'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.5.0'])

# 8.5.3
UGE_RELEASE_OBJECT_MAP['8.5.3'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.5.0'])

# 8.5.3_C104_1
UGE_RELEASE_OBJECT_MAP['8.5.3_C104_1'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.5.3'])
UGE_RELEASE_OBJECT_MAP['8.5.3_C104_1']['ParallelEnvironment'] = '2.0'

# 8.5.4
UGE_RELEASE_OBJECT_MAP['8.5.4'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.5.0'])

# 8.5.4_C104_1
UGE_RELEASE_OBJECT_MAP['8.5.4_C104_1'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.5.4'])
UGE_RELEASE_OBJECT_MAP['8.5.4_C104_1']['ParallelEnvironment'] = '2.0'

# 8.5.4_C104_2
UGE_RELEASE_OBJECT_MAP['8.5.4_C104_2'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.5.4_C104_1'])

# 8.5.4_C104_3
UGE_RELEASE_OBJECT_MAP['8.5.4_C104_3'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.5.4_C104_2'])

# 8.5.4_C104_4
UGE_RELEASE_OBJECT_MAP['8.5.4_C104_4'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.5.4_C104_3'])

# 8.5.4_C104_5
UGE_RELEASE_OBJECT_MAP['8.5.4_C104_5'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.5.4_C104_4'])

# 8.5.4_C104_6
UGE_RELEASE_OBJECT_MAP['8.5.4_C104_6'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.5.4_C104_5'])

# 8.5.5
UGE_RELEASE_OBJECT_MAP['8.5.5'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.5.0'])

# 8.5.6
UGE_RELEASE_OBJECT_MAP['8.5.6'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.5.0'])

# 8.6.0 (with affinity support)
UGE_RELEASE_OBJECT_MAP['8.6.0'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.5.0'])
UGE_RELEASE_OBJECT_MAP['8.6.0']['ClusterQueue'] = '2.0'
UGE_RELEASE_OBJECT_MAP['8.6.0']['ComplexConfiguration'] = '3.0'
UGE_RELEASE_OBJECT_MAP['8.6.0']['SchedulerConfiguration'] = '2.0'
UGE_RELEASE_OBJECT_MAP['8.6.0']['ParallelEnvironment'] = '2.0'
UGE_RELEASE_OBJECT_MAP['8.6.0']['JobClass'] = '3.0'

# 8.6.1
UGE_RELEASE_OBJECT_MAP['8.6.1'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.6.0'])

# 8.6.2
UGE_RELEASE_OBJECT_MAP['8.6.2'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.6.1'])

# 8.6.3
UGE_RELEASE_OBJECT_MAP['8.6.3'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.6.2'])

# 8.6.4
UGE_RELEASE_OBJECT_MAP['8.6.4'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.6.3'])

# 8.6.5
UGE_RELEASE_OBJECT_MAP['8.6.5'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.6.4'])

# 8.6.6
UGE_RELEASE_OBJECT_MAP['8.6.6'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.6.5'])

# 8.6.7
UGE_RELEASE_OBJECT_MAP['8.6.7prealpha'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.6.6'])
UGE_RELEASE_OBJECT_MAP['8.6.7alpha1'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.6.6'])
UGE_RELEASE_OBJECT_MAP['8.6.7'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.6.6'])

# 8.6.8
UGE_RELEASE_OBJECT_MAP['8.6.8prealpha'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.6.7'])
UGE_RELEASE_OBJECT_MAP['8.6.8'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.6.7'])

# 8.6.9
UGE_RELEASE_OBJECT_MAP['8.6.9prealpha'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.6.8'])
UGE_RELEASE_OBJECT_MAP['8.6.9'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.6.8'])

# 8.6.10
UGE_RELEASE_OBJECT_MAP['8.6.10prealpha'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.6.9'])

# 8.7.0
UGE_RELEASE_OBJECT_MAP['8.7.0alpha'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.6.0'])
UGE_RELEASE_OBJECT_MAP['8.7.0alpha']['SchedulerConfiguration'] = '3.0'
UGE_RELEASE_OBJECT_MAP['8.7.0alpha']['ComplexConfiguration'] = '4.0'
UGE_RELEASE_OBJECT_MAP['8.7.0alpha']['JobClass'] = '4.0'
UGE_RELEASE_OBJECT_MAP['8.7.0alpha2'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.7.0alpha'])
UGE_RELEASE_OBJECT_MAP['8.7.0beta'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.7.0alpha2'])
UGE_RELEASE_OBJECT_MAP['8.7.0beta2'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.7.0beta'])
UGE_RELEASE_OBJECT_MAP['8.7.0beta3'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.7.0beta2'])
UGE_RELEASE_OBJECT_MAP['8.7.0beta4'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.7.0beta3'])
UGE_RELEASE_OBJECT_MAP['8.7.0beta5'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.7.0beta4'])
UGE_RELEASE_OBJECT_MAP['8.7.0beta6'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.7.0beta5'])
UGE_RELEASE_OBJECT_MAP['8.7.0beta7'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.7.0beta6'])
UGE_RELEASE_OBJECT_MAP['8.7.0beta8'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.7.0beta7'])
UGE_RELEASE_OBJECT_MAP['8.7.0beta9'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.7.0beta8'])
UGE_RELEASE_OBJECT_MAP['8.7.0'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.7.0beta9'])

# 8.7.1
UGE_RELEASE_OBJECT_MAP['8.7.1prealpha'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.7.0'])
UGE_RELEASE_OBJECT_MAP['8.7.1'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.7.1prealpha'])

# 8.7.2
UGE_RELEASE_OBJECT_MAP['8.7.2prealpha'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.7.1'])
UGE_RELEASE_OBJECT_MAP['8.7.2prealpha2'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.7.1'])
UGE_RELEASE_OBJECT_MAP['8.7.2prealpha3'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.7.1'])
# if new version string shall be used (see uge/api/impl/*executor.py#get_uge_version)
# UGE_RELEASE_OBJECT_MAP['2022.1.0pre3'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.7.0'])

UGE_RELEASE_OBJECT_MAP['8.7.2'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.7.1'])

# 8.7.3
UGE_RELEASE_OBJECT_MAP['8.7.3'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.7.2'])

# 8.8.0
UGE_RELEASE_OBJECT_MAP['8.8.0'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.7.0'])

# 8.8.1
UGE_RELEASE_OBJECT_MAP['8.8.1'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.8.0'])

# 8.9.0
UGE_RELEASE_OBJECT_MAP['8.9.0'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.8.1'])

# 8.9.1
UGE_RELEASE_OBJECT_MAP['8.9.1prealpha'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.9.0'])
UGE_RELEASE_OBJECT_MAP['8.9.1'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.9.0'])

# 8.10.0
UGE_RELEASE_OBJECT_MAP['8.10.0prealpha'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.9.1'])
UGE_RELEASE_OBJECT_MAP['8.10.0beta'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.9.1'])
UGE_RELEASE_OBJECT_MAP['8.10.0'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.9.1'])

# 8.11.0
UGE_RELEASE_OBJECT_MAP['8.11.0prealpha'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.10.0'])
UGE_RELEASE_OBJECT_MAP['8.11.0'] = copy.copy(UGE_RELEASE_OBJECT_MAP['8.10.0'])



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
import errno
import re
import copy
import os
import shutil
import tempfile
from uge.log.log_manager import LogManager
from uge.exceptions.object_not_found import ObjectNotFound
from uge.exceptions.object_already_exists import ObjectAlreadyExists
from uge.exceptions.invalid_argument import InvalidArgument
from uge.exceptions.invalid_request import InvalidRequest
from uge.objects.qconf_object_factory import QconfObjectFactory
from uge.objects.qconf_object import QconfObject
from uge.objects.qconf_name_list import QconfNameList


class DictBasedObjectManager(object):
    QCONF_ERROR_REGEX_LIST = [
        (re.compile('.*is still referenced in.*'), InvalidRequest),
        (re.compile('.*does not exist.*'), ObjectNotFound),
        (re.compile('.*no .* list defined.*'), ObjectNotFound),
        (re.compile('.*is not known as.*'), ObjectNotFound),
    ]

    # Failure incorrectly classified as successful outcome
    QCONF_FAILURE_REGEX_LIST = []

    GENERATE_OBJECT_FACTORY_METHOD = None
    GENERATE_BULK_OBJECT_FACTORY_METHOD = None
    OBJECT_NAME_KEY = None
    OBJECT_CLASS_NAME = None
    OBJECT_CLASS_UGE_NAME = None
    OBJECT_CLASS_UGE_LIST_DETAILS_NAME = None

    BULK_SEPARATOR = "^=================+"
    KEY_VALUE_DELIMITER = ' '

    DEFAULT_LIST_DELIMITER = ','

    def __init__(self, qconf_executor):
        self.logger = LogManager.get_instance().get_logger(self.__class__.__name__)
        self.qconf_executor = qconf_executor
        self.object_dump_ignored_key_list = []

    def generate_object(self, name=None, data=None, metadata=None,
                        json_string=None, uge_version=None,
                        add_required_data=True):
        if not uge_version:
            uge_version = self.qconf_executor.get_uge_version()
        generated_object = self.GENERATE_OBJECT_FACTORY_METHOD(
            uge_version, name=name, data=data, metadata=metadata,
            json_string=json_string, add_required_data=add_required_data)
        return generated_object

    def __prepare_object(self, pycl_object=None, name=None, data=None,
                         metadata=None, json_string=None,
                         add_required_data=True):
        uge_version = self.qconf_executor.get_uge_version()
        data2 = copy.copy(data)
        metadata2 = copy.copy(metadata)
        name2 = name
        # If pycl_object is provided, combine its data/metadata
        # with provided data/metadata
        if pycl_object is not None:
            generated_object = self.GENERATE_OBJECT_FACTORY_METHOD(uge_version, add_required_data=False)
            if not str(type(pycl_object)) == str(type(generated_object)):
                raise InvalidArgument(
                    'The pycl_object argument must be an instance of %s.' % generated_object.__class__.__name__)
            data2 = copy.copy(pycl_object.data)
            if data:
                data2.update(data)
            metadata2 = copy.copy(pycl_object.metadata)
            if metadata:
                metadata2.update(metadata)
            if not name2:
                name2 = pycl_object.name
        result_object = self.GENERATE_OBJECT_FACTORY_METHOD(
            uge_version, name=name2, data=data2, metadata=metadata2,
            json_string=json_string, add_required_data=add_required_data)
        result_object.check_user_provided_keys()
        return result_object

    def verify_object_before_add(self, pycl_object):
        return

    def add_object(self, pycl_object=None, name=None, data=None,
                   metadata=None, json_string=None):
        new_object = self.__prepare_object(
            pycl_object=pycl_object, name=name, data=data,
            metadata=metadata, json_string=json_string)
        if self.OBJECT_NAME_KEY:
            object_name = new_object.data.get(self.OBJECT_NAME_KEY)
        elif new_object.name:
            object_name = new_object.name
        else:
            object_name = ''
        try:
            old_object = self.get_object(object_name)
            raise ObjectAlreadyExists('%s %s already exists.' % (self.OBJECT_CLASS_NAME, object_name))
        except ObjectNotFound:
            # ok
            pass
        new_object.remove_optional_keys()
        self.verify_object_before_add(new_object)
        self.qconf_executor.execute_qconf_with_object('-A%s' % self.OBJECT_CLASS_UGE_NAME, new_object,
                                                      self.QCONF_ERROR_REGEX_LIST)
        new_object.set_add_metadata()
        return new_object

    def add_objects(self, object_list, dirname=None):
        if not dirname:
            dirname = tempfile.mktemp()
        self.mk_object_dir(dirname)
        self.write_objects(object_list, dirname)
        self.add_objects_from_dir(dirname)
        self.rm_object_dir(dirname)

    def add_objects_from_dir(self, dirname):
        self.qconf_executor.execute_qconf_with_dir('-A%s' % self.OBJECT_CLASS_UGE_NAME, dirname,
                                                   self.QCONF_ERROR_REGEX_LIST)

    def verify_object_before_modify(self, pycl_object):
        return

    def modify_object(self, pycl_object=None, name=None, data=None,
                      metadata=None, json_string=None):
        generated_object = self.__prepare_object(
            pycl_object=pycl_object, name=name, data=data,
            metadata=metadata, json_string=json_string,
            add_required_data=False)
        if self.OBJECT_NAME_KEY:
            object_name = generated_object.data.get(self.OBJECT_NAME_KEY)
        elif generated_object.name:
            object_name = generated_object.name
        else:
            object_name = ''
        updated_object = self.get_object(object_name)
        updated_object.data.update(generated_object.data)
        updated_object.remove_optional_keys()
        self.verify_object_before_modify(updated_object)
        self.qconf_executor.execute_qconf_with_object('-M%s' % self.OBJECT_CLASS_UGE_NAME, updated_object,
                                                      self.QCONF_ERROR_REGEX_LIST)
        updated_object.set_modify_metadata()
        return updated_object

    def modify_objects(self, object_list, dirname=None):
        if not dirname:
            dirname = tempfile.mktemp()
        self.mk_object_dir(dirname)
        self.write_objects(object_list, dirname)
        self.modify_objects_from_dir(dirname)
        self.rm_object_dir(dirname)
        return

    def modify_objects_from_dir(self, dir):
        self.qconf_executor.execute_qconf_with_dir('-M%s' % self.OBJECT_CLASS_UGE_NAME, dir,
                                                   self.QCONF_ERROR_REGEX_LIST)
        return

    def replace_object(self, updated_object):
        self.qconf_executor.execute_qconf_with_object('-M%s' % self.OBJECT_CLASS_UGE_NAME, updated_object,
                                                      self.QCONF_ERROR_REGEX_LIST)
        updated_object.set_modify_metadata()
        return updated_object

    def get_object(self, name):
        uge_version = self.qconf_executor.get_uge_version()
        retrieved_object = self.GENERATE_OBJECT_FACTORY_METHOD(uge_version, add_required_data=False)
        qconf_output = self.qconf_executor.execute_qconf('-s%s %s' % (self.OBJECT_CLASS_UGE_NAME, name),
                                                         self.QCONF_ERROR_REGEX_LIST,
                                                         failure_regex_list=self.QCONF_FAILURE_REGEX_LIST).get_stdout()
        retrieved_object.set_data_dict_from_qconf_output(qconf_output)
        retrieved_object.name = name
        return retrieved_object

    def verify_object_before_delete(self, pycl_object):
        return

    def delete_object(self, name):
        deleted_object = self.get_object(name)
        self.verify_object_before_delete(deleted_object)
        self.qconf_executor.execute_qconf('-d%s %s' % (self.OBJECT_CLASS_UGE_NAME, name), self.QCONF_ERROR_REGEX_LIST)

    def delete_object_list(self, object_list, dirname=None):
        if not dirname:
            dirname = tempfile.mktemp()
        self.mk_object_dir(dirname)
        self.write_objects(object_list, dirname)
        self.delete_objects_from_dir(dirname)
        self.rm_object_dir(dirname)
        return

    def delete_objects_from_dir(self, dir):
        self.qconf_executor.execute_qconf_with_dir('-D%s' % self.OBJECT_CLASS_UGE_NAME, dir,
                                                   self.QCONF_ERROR_REGEX_LIST)
        return

    def list_objects(self):
        try:
            qconf_output = self.qconf_executor.execute_qconf('-s%sl' % (self.OBJECT_CLASS_UGE_NAME),
                                                             self.QCONF_ERROR_REGEX_LIST).get_stdout()
            object_list = QconfNameList(metadata={'description': 'List of %s object names' % (self.OBJECT_CLASS_NAME)},
                                        data=QconfObject.get_list_from_qconf_output(qconf_output))
        except ObjectNotFound as ex:
            object_list = QconfNameList(metadata={'description': 'List of %s object names' % (self.OBJECT_CLASS_NAME)},
                                        data=[])
        return object_list

    def get_objects(self):
        bulk_output = self.qconf_executor.execute_qconf(
            '-s%s%s' % (self.OBJECT_CLASS_UGE_NAME, self.OBJECT_CLASS_UGE_LIST_DETAILS_NAME),
            self.QCONF_ERROR_REGEX_LIST, failure_regex_list=self.QCONF_FAILURE_REGEX_LIST).get_stdout()
        bulk_object = self.parse_bulk_output(bulk_output)
        return bulk_object

    def get_bulk_dump_filename(self, object):
        return ''

    def write_objects(self, object_list, dirname):
        for object in object_list:
            filename = self.get_bulk_dump_filename(object)
            lines = ''
            for key, value in list(object.data.items()):
                if key in self.object_dump_ignored_key_list:
                    continue
                lines += '%s%s%s\n' % (key, self.KEY_VALUE_DELIMITER, object.py_to_uge(key, value))
            with open(os.path.join(dirname, filename), 'w') as dumpfile:
                dumpfile.write(lines)
        return

    def mk_object_dir(self, path):
        try:
            os.makedirs(path)
        except OSError as exception:
            if exception.errno != errno.EEXIST:
                raise
        return

    def rm_object_dir(self, dirname):
        shutil.rmtree(dirname)
        return

    def delete_objects(self, name_list):
        names = self.DEFAULT_LIST_DELIMITER.join(name_list)
        self.qconf_executor.execute_qconf('-d%s %s' % (self.OBJECT_CLASS_UGE_NAME, names), self.QCONF_ERROR_REGEX_LIST)
        return

    def parse_bulk_output(self, bulk_output):
        uge_version = self.qconf_executor.get_uge_version()
        object_list = []
        lines = []
        # print "bulk_output begin"
        # print bulk_output
        # print "bulk_output end"
        # print "len(bulk_output)=",len(bulk_output)
        if len(bulk_output) > 0:
            lines = bulk_output.split('\n')
        if len(lines) > 0:
            retrieved_object = self.GENERATE_OBJECT_FACTORY_METHOD(uge_version, add_required_data=False)
            object_list.append(retrieved_object)
        # Parse lines until a separator is found, and then create new dictionary object
        for line in lines:
            if not line:
                continue
            if self.BULK_SEPARATOR:
                if re.match(self.BULK_SEPARATOR, line):
                    retrieved_object = self.GENERATE_OBJECT_FACTORY_METHOD(uge_version, add_required_data=False)
                    object_list.append(retrieved_object)
                    continue;
            key_value = line.split(self.KEY_VALUE_DELIMITER)
            key = key_value[0]
            value = self.KEY_VALUE_DELIMITER.join(key_value[1:]).strip()
            retrieved_object.data[key] = retrieved_object.uge_to_py(key, value)

        return object_list


#############################################################################
# Testing.
if __name__ == '__main__':
    pass



#!/usr/bin/env python
# 
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
# 
from .utils import needs_uge
from .utils import create_config_file
from .utils import generate_random_string

from uge.api.qconf_api import QconfApi
from uge.config.config_manager import ConfigManager
from uge.log.log_manager import LogManager
from uge.exceptions.object_not_found import ObjectNotFound
from uge.exceptions.object_already_exists import ObjectAlreadyExists

create_config_file()
API = QconfApi()
OPERATOR_NAME = '%s' % generate_random_string(6)
CONFIG_MANAGER = ConfigManager.get_instance()
LOG_MANAGER = LogManager.get_instance()


@needs_uge
def test_list_operators():
    ol = API.list_operators()
    assert (ol is not None)


def test_add_operator():
    ol = API.list_operators()
    ol2 = API.add_operators([OPERATOR_NAME])
    assert (len(ol2) == len(ol) + 1)
    assert (ol2.count(OPERATOR_NAME) == 1)


def test_delete_operator():
    ol = API.list_operators()
    ol2 = API.delete_operators([OPERATOR_NAME])
    assert (len(ol2) == len(ol) - 1)
    assert (ol2.count(OPERATOR_NAME) == 0)


def test_object_already_exists():
    API.add_operators([OPERATOR_NAME])
    try:
        API.add_operators([OPERATOR_NAME])
        assert (False)
    except ObjectAlreadyExists as ex:
        # ok
        pass
    API.delete_operators([OPERATOR_NAME])


def test_object_not_found():
    try:
        API.delete_operators([OPERATOR_NAME])
    except ObjectNotFound as ex:
        # ok
        pass



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
from .qconf_dict_list import QconfDictList
from .qconf_object import QconfObject


class ShareTree(QconfDictList):
    """ This class encapsulates UGE share tree object. """

    #: Object version.
    VERSION = '1.0'

    #: Key that designates start of an object in a list
    FIRST_KEY = 'id'

    #: Object name key.
    NAME_KEY = None

    #: Object keys that must be provided by user.
    USER_PROVIDED_KEYS = ['name']

    #: Default values for required dictionary data keys.
    REQUIRED_DATA_DEFAULTS = {
        'id': 0,
        'name': None,
        'type': 1,
        'shares': 0,
        'childnodes': None,

    }

    BOOL_KEY_MAP = QconfObject.get_bool_key_map(REQUIRED_DATA_DEFAULTS)
    INT_KEY_MAP = QconfObject.get_int_key_map(REQUIRED_DATA_DEFAULTS)
    FLOAT_KEY_MAP = QconfObject.get_float_key_map(REQUIRED_DATA_DEFAULTS)
    DEFAULT_LIST_DELIMITER = ','

    def __init__(self, data=None, metadata=None, json_string=None):
        """ 
        Class constructor. 

        :param data: Configuration data. If provided, it will override corresponding data from JSON string representation.
        :type data: dict

        :param metadata: Configuration metadata. If provided, it will override corresponding metadata from JSON string representation.
        :type metadata: dict

        :param json_string: Configuration JSON string representation.
        :type json_string: str

        :raises: **InvalidArgument** - in case metadata is not a dictionary, JSON string is not valid, or it does not represent a ShareTree object.
        """
        QconfDictList.__init__(self, data=data, metadata=metadata, json_string=json_string)

    def to_uge(self):
        """ 
        Converts object to string acceptable as input for UGE qconf command.

        :returns: Object's UGE-formatted string.
        """
        lines = ''
        for d in self.data:
            for key in ['id', 'name', 'type', 'shares', 'childnodes']:
                value = d.get(key)
                lines += '%s%s%s\n' % (key, self.KEY_VALUE_DELIMITER, self.py_to_uge(key, value))
        return lines


#############################################################################
# Testing.
if __name__ == '__main__':
    pass



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
import re
import xmltodict
import json

from uge.utility.uge_subprocess import UgeSubprocess
from uge.log.log_manager import LogManager
from uge.exceptions.ar_exception import AdvanceReservationException
from uge.exceptions.command_failed import CommandFailed
from uge.exceptions.qmaster_unreachable import QmasterUnreachable
from uge.exceptions.authorization_error import AuthorizationError
from uge.exceptions.object_not_found import ObjectNotFound


class QrstatExecutor(object):
    QRSTAT_ERROR_REGEX_LIST = [
        (re.compile('.*unable to send message to qmaster.*'), QmasterUnreachable),
        (re.compile('.*must be manager.*'), AuthorizationError),
        (re.compile('denied.*'), AuthorizationError),
        (re.compile('.*does not exist.*'), ObjectNotFound),
        (re.compile('.*no.*defined.*'), ObjectNotFound),
        (re.compile('.*'), AdvanceReservationException)
    ]
    QRSTAT_SUCCESS_REGEX_LIST = []  # for successful outcome incorrectly classified as failure
    QRSTAT_FAILURE_REGEX_LIST = []  # for failure incorrectly classified as successful outcome

    def __init__(self, sge_root, sge_cell, sge_qmaster_port, sge_execd_port):
        self.logger = LogManager.get_instance().get_logger(self.__class__.__name__)
        self.env_dict = {
            'SGE_ROOT': sge_root,
            'SGE_CELL': sge_cell,
            'SGE_QMASTER_PORT': str(sge_qmaster_port),
            'SGE_EXECD_PORT': str(sge_execd_port),
            'SGE_SINGLE_LINE': '1',
        }
        self.uge_version = None
        self.__configure()

    def __configure(self):
        self.logger.trace('Retrieving UGE version')
        uge_version = self.get_uge_version()
        self.logger.debug('UGE version: %s' % uge_version)

    def get_uge_version(self):
        if not self.uge_version:
            p = self.execute_qrstat('-help')
            lines = p.get_stdout().split('\n')
            if not len(lines):
                raise AdvanceReservationException('Cannot determine UGE version from output: %s' % p.get_stdout())
            self.uge_version = lines[0].split()[-1].split("_")[0].replace('(','').replace(')','')
            # if 2022.0.1 is used instead
            # self.uge_version = lines[0].split()[1]
        return self.uge_version

    def execute_qrstat(self, cmd, error_regex_list=[], error_details=None, combine_error_lines=False,
                       success_regex_list=[], failure_regex_list=[]):
        try:
            command = '. %s/%s/common/settings.sh; qrstat %s' % (
            self.env_dict['SGE_ROOT'], self.env_dict['SGE_CELL'], cmd)
            p = UgeSubprocess(command, env=self.env_dict)
            p.run()

            # In some cases successful outcome is actually a failure
            error = p.get_stderr()
            if error:
                for (pattern, qrstatExClass) in failure_regex_list + QrstatExecutor.QRSTAT_FAILURE_REGEX_LIST:
                    if pattern.match(error):
                        raise qrstatExClass(error, error_details=error_details)
            return p
        except CommandFailed as ex:
            error = str(ex)
            if combine_error_lines:
                error = error.replace('\n', '; ')
            for (pattern, result) in success_regex_list + QrstatExecutor.QRSTAT_SUCCESS_REGEX_LIST:
                if pattern.match(error):
                    self.logger.debug(
                        'Ignoring command failed for success pattern, replacing stdout with result: "%s"' % result)
                    p.stdout_ = result
                    return p
            for (pattern, qrstatExClass) in error_regex_list + QrstatExecutor.QRSTAT_ERROR_REGEX_LIST:
                if pattern.match(error):
                    raise qrstatExClass(error, error_details=error_details)
            raise

    def get_ar(self, name):
        p = self.execute_qrstat('-json -ar ' + name)
        lines = p.get_stdout()
        aro = json.loads(lines)
        return aro

    def get_ar_summary(self):
        p = self.execute_qrstat('-json')
        lines = p.get_stdout()
        aro = json.loads(lines)
        return aro

    def get_ar_list(self):
        p = self.execute_qrstat('-json -u "*"')
        lines = p.get_stdout()
        aro = json.loads(lines)
        arl = []
        if 'ar_summary' in aro['qrstat']:
            for item in aro['qrstat']['ar_summary']:
                arl.append(item['id'])
        return arl


#############################################################################
# Testing.
if __name__ == '__main__':
    from uge.exceptions.command_failed import CommandFailed

    executor = QrstatExecutor(sge_root='/Users/aalefeld/univa/clusters/UGE86', sge_cell='default',
                              sge_qmaster_port=5560, sge_execd_port=5561)
    try:
        print('Version: ', executor.get_uge_version())

        p = executor.execute_qrstat('-help')
        print(p.get_stdout())
        print(p.get_exit_status())

        # p = executor.execute_qrstat("-xml")
        p = executor.execute_qrstat("-xml -ar 311")
        s = p.get_stdout()
        o = xmltodict.parse(s)
        print(json.dumps(o))
        print(p.get_exit_status())

    except CommandFailed as ex:
        print('Exit Status: ', ex.get_command_exit_status())
        print('Std Error  : ', ex.get_command_stderr())



#!/usr/bin/env python
#
# ___INFO__MARK_BEGIN__
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__
#
import re
from uge.exceptions.object_not_found import ObjectNotFound
from uge.exceptions.invalid_request import InvalidRequest
from uge.objects.qconf_object_factory import QconfObjectFactory
from .dict_based_object_manager import DictBasedObjectManager


class CalendarManager(DictBasedObjectManager):
    QCONF_ERROR_REGEX_LIST = [
        (re.compile('.*is still referenced in.*'), InvalidRequest),
        (re.compile('.*error in disabled_.*'), InvalidRequest),
        (re.compile('.*does not exist.*'), ObjectNotFound),
        (re.compile('.*is not a calendar.*'), ObjectNotFound),
        (re.compile('.*no calendar defined.*'), ObjectNotFound),
    ]

    # Failure incorrectly classified as successful outcome
    QCONF_FAILURE_REGEX_LIST = []

    GENERATE_OBJECT_FACTORY_METHOD = QconfObjectFactory.generate_calendar
    OBJECT_NAME_KEY = 'calendar_name'
    OBJECT_CLASS_NAME = 'Calendar'
    OBJECT_CLASS_UGE_NAME = 'cal'
    OBJECT_CLASS_UGE_LIST_DETAILS_NAME = 'ld'

    def __init__(self, qconf_executor):
        DictBasedObjectManager.__init__(self, qconf_executor)

    def get_bulk_dump_filename(self, object):
        return 'conf_api_dump_' + object.data['calendar_name']

#############################################################################
# Testing.
if __name__ == '__main__':
    pass



# 
# ___INFO__MARK_BEGIN__ 
#######################################################################################
# Copyright 2016-2024 Altair Engineering Inc.
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License.
#
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#######################################################################################
# ___INFO__MARK_END__ 
# 
from __future__ import with_statement, print_function
import sys

# Work around http://bugs.python.org/issue15881#msg170215
try:
    import multiprocessing
except ImportError:
    pass

try:
    from setuptools import setup
    extra=dict(include_package_data=True)
except ImportError:
    from distutils.core import setup
    extra={}

import os

from uge import __version__

if sys.version_info <= (2, 5):
    error="ERROR: UGE PyCL requires Python Version 2.6 or above...exiting."
    print(error, file=sys.stderr)
    sys.exit(1)

with open("README.rst") as f:
    long_description=f.read()

setup(name='uge-pycl',
      version=__version__,
      description='UGE Python Configuration Library',
      long_description=long_description,
      # long_description_content_type='text/markdown',
      author='Altair Engineering Inc.',
      author_email='support@altair.com',
      test_suite='test',
      entry_points={
        'console_scripts': [
            'qconf-convert=uge.cli.qconf_convert:run',
        ],
      },
      url='https://www.altair.com',
      packages=[
                'uge', 
                'uge/api', 
                'uge/api/impl', 
                'uge/cli', 
                'uge/config', 
                'uge/constants', 
                'uge/exceptions', 
                'uge/log', 
                'uge/objects', 
                'uge/utility'
      ],
      install_requires=['decorator','xmltodict'],
      package_data={
      },
      license='Apache 2.0',
      platforms='Posix; MacOS X',
      classifiers=[
                     'Intended Audience :: Developers',
                     'Operating System :: OS Independent',
                     'Programming Language :: Python :: 2',
                     'Programming Language :: Python :: 2.6',
                     'Programming Language :: Python :: 2.7',
                     'Programming Language :: Python :: 3',
                     ],
      options={'bdist_wheel': {'universal':True}},
      **extra
      )
